#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""tests for PrepareOptionFiles."""
from __future__ import print_function
from __future__ import absolute_import
import unittest
import os
import filecmp
import copy
import collections
import xml.etree.ElementTree as ET
from mock import mock_open, patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Core.Utilities import PrepareOptionFiles
from ILCDIRAC.Core.Utilities.PrepareOptionFiles import prepareMacFile
from Tests.Utilities.FileUtils import FileUtil
from Tests.Utilities.GeneralUtils import assertEqualsXml, \
    assertEqualsImproved, assertDiracFailsWith, assertDiracSucceeds, \
    assertDiracSucceedsWith_equals
from six.moves import range
from six.moves import zip

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Core.Utilities.PrepareOptionFiles'

# TODO split up in separate classes (if too much time, decent for now. Quite
# some code duplication but hard to remove + unlikely&not too much effort to change)

# pylint: disable=no-member, too-many-public-methods


class TestPrepareOptionsFile(unittest.TestCase):
  """tests of PrepareOptionsFile."""

  def setUp(self):
    """Make fake files for the test."""
    self.referenceMacfile = "ref.mac"
    self.inputmac = "input.mac"
    self.outputmac = "output.mac"
    self.stdhep = "dummy.stdhep"
    self.nbevts = 123
    self.startfrom = 12

  def tearDown(self):
    """Remove the fake files."""
    for rFile in (self.outputmac, self.inputmac, self.referenceMacfile):
      try:
        os.remove(rFile)
      except OSError as e:
        print("Error removing %s: %s " % (rFile, str(e)))

  def createInputMacFile(self):
    """create the input macfile for the test."""
    lines = []
    lines.append("/generator/filename asads")
    lines.append("/generator/skipEvents 123")
    lines.append("/random/seed 321")
    lines.append("/run/beamOn 456")
    lines.append("/lcdd/url http://google.com")
    lines.append("/lcio/filename outfile.wrong")
    lines.append("/some/other/command")
    lines.append("/do/not/remove/this")

    with open(self.inputmac, "w") as ifile:
      ifile.write("\n".join(lines))
      ifile.write("\n")

  def createReferenceMacFile(self, startfrom=0, nbevents=10, outputfile=None,
                              detector=None, randomseed=0):
    """create the input macfile for the test."""
    lines = []
    if detector:
      lines.append("/lcdd/url %s.lcdd" % detector)
    if outputfile:
      lines.append("/lcio/filename %s" % outputfile)
    lines.append("/lcio/runNumber %d" % randomseed)
    if not detector:
      lines.append("/lcdd/url http://google.com")
      lines.append("")
    if not outputfile:
      lines.append("/lcio/filename outfile.wrong")
      lines.append("")
    lines.append("/some/other/command")
    lines.append("")
    lines.append("/do/not/remove/this")
    lines.append("")
    lines.append("/generator/filename dummy.stdhep")
    lines.append("/generator/skipEvents %d" % startfrom)
    lines.append("/random/seed %d" % randomseed)
    lines.append("/run/beamOn %d" % nbevents)

    with open(self.referenceMacfile, "w") as ifile:
      ifile.write("\n".join(lines))
      ifile.write("\n")

  def compareMacFiles(self):
    """create the new macfile with the expected one."""
    res = filecmp.cmp(self.outputmac, self.referenceMacfile)
    print("Compare mac files", res)
    return res

  def test_prepMacFile1(self):
    """test with start, events, stdhep.............................................................."""

    self.createInputMacFile()
    startfrom = 10
    nbevents = 10
    stdhep = "dummy.stdhep"
    self.createReferenceMacFile(startfrom=startfrom, nbevents=nbevents)
    prepareMacFile(self.inputmac, self.outputmac, stdhep=stdhep, nbevts=nbevents,
                   startfrom=startfrom)
    self.assertTrue(self.compareMacFiles())

  def test_prepMacFile2(self):
    """test with start, events, stdhep and outputfile..............................................."""
    self.createInputMacFile()
    startfrom = 10
    nbevents = 10
    stdhep = "dummy.stdhep"
    outputfile = "out1.slcio"
    self.createReferenceMacFile(startfrom=startfrom, nbevents=nbevents,
                                 outputfile=outputfile)
    prepareMacFile(self.inputmac, self.outputmac, stdhep=stdhep, nbevts=nbevents,
                    startfrom=startfrom, outputlcio=outputfile)
    self.assertTrue(self.compareMacFiles())

  def test_prepMacFile3(self):
    """test with start, events, stdhep and detector................................................."""
    self.createInputMacFile()
    startfrom = 10
    nbevents = 10
    stdhep = "dummy.stdhep"
    outputfile = "out1.slcio"
    detector = "CLIC_SID_CDR"
    self.createReferenceMacFile(startfrom=startfrom,
                                 nbevents=nbevents,
                                 outputfile=outputfile,
                                 detector=detector
                               )
    prepareMacFile(self.inputmac, self.outputmac, stdhep=stdhep, nbevts=nbevents,
                    startfrom=startfrom, outputlcio=outputfile, detector=detector)
    self.assertTrue(self.compareMacFiles())

  def test_prepMacFile4(self):
    """test with start, events, stdhep and randomseed..............................................."""
    self.createInputMacFile()
    startfrom = 10
    nbevents = 10
    stdhep = "dummy.stdhep"
    randomseed = 123
    self.createReferenceMacFile(startfrom=startfrom, nbevents=nbevents,
                                 randomseed=randomseed)
    prepareMacFile(self.inputmac, self.outputmac, stdhep=stdhep, nbevts=nbevents,
                    startfrom=startfrom, randomseed=randomseed)
    self.assertTrue(self.compareMacFiles())


class TestPrepareOptionsFilePatch(unittest.TestCase):
  """Class for tests that share several mocks."""
  teststr = """
<?xml version="1.0" encoding="us-ascii"?>
<!-- ?xml-stylesheet type="text/xsl" href="http://ilcsoft.desy.de/marlin/marlin.xsl"? -->
<!-- ?xml-stylesheet type="text/xsl" href="marlin.xsl"? -->

<marlin xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="http://ilcsoft.desy.de/marlin/marlin.xsd">

   <execute>
      <processor name="MyTomatoProcessor"/>
   </execute>

   <global>
      <parameter name="Verbosity" value="ERROR"/>
   </global>

 <processor name="MyTomatoProcessor" type="TomatoProcessor">
 <!--Automated analysis-->
  <!--Name of the MCParticle collection-->
  <parameter name="MCCollectionName" type="string" lcioInType="MCParticle"> MCParticle </parameter>
  <!--Root OutputFile-->
  <parameter name="OutputFile" type="string" value="tomato.root"/>
  <!--verbosity level of this processor ("DEBUG0-4,MESSAGE0-4,WARNING0-4,ERROR0-4,SILENT")-->
  <!--parameter name="Verbosity" type="string" value=""/-->
</processor>

</marlin>
    """

  def setUp(self):
    self.dep1 = {'app': True, 'version': True}
    self.dep2 = {'app': True, 'version': True}
    self.dep3 = {'app': True, 'version': True}

  def test_getnewldlibs_cornercase(self):
    # TODO: Understand method: Currently this method ignores every library
    # path except the last one in the list and just ignores if
    # getSoftwareFolder fails
    reference = os.environ.get('LD_LIBRARY_PATH', '')
    if 'LD_LIBRARY_PATH' in os.environ:
      with patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(side_effect=[S_ERROR(), S_OK('aFolder'), S_OK('bFolder')])), \
          patch("%s.resolveDeps" % MODULE_NAME, new=Mock(return_value=[self.dep1, self.dep2, self.dep3])), \
          patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[True, False, False, True])), \
          patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=True)) as mock_removelibc:
        result = PrepareOptionFiles.getNewLDLibs(None, None, None)
    else:
      with patch.dict('os.environ', {'LD_LIBRARY_PATH': ''}), \
          patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(side_effect=[S_ERROR(), S_OK('aFolder'), S_OK('bFolder')])), \
          patch("%s.resolveDeps" % MODULE_NAME, new=Mock(return_value=[self.dep1, self.dep2, self.dep3])), \
          patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[True, False, False, True])), \
          patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=True)) as mock_removelibc:
        result = PrepareOptionFiles.getNewLDLibs(None, None, None)
    self.assertEqual("%s:%s" % ('bFolder/LDLibs', reference), result)
    mock_removelibc.assert_any_call("aFolder/lib")
    mock_removelibc.assert_any_call("bFolder/LDLibs")

  def test_getnewldlibs_nochange(self):
    reference = os.environ.get('LD_LIBRARY_PATH', '')
    with patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(side_effect=[S_ERROR(), S_OK(''), S_OK('')])), \
         patch("%s.resolveDeps" % MODULE_NAME, new=Mock(return_value=[self.dep1, self.dep2, self.dep3])), \
         patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[False, False, False, False])), \
         patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=True)):
      self.assertEqual(reference, PrepareOptionFiles.getNewLDLibs(None, None,
                                                                   None))

  def test_getnewldlibs_noldlibpath(self):
    with patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(side_effect=[S_ERROR(), S_OK('basetest'), S_ERROR()])), \
        patch("%s.resolveDeps" % MODULE_NAME, new=Mock(return_value=[self.dep1, self.dep2, self.dep3])), \
        patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[True, False, False, False])), \
        patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=True)), \
        patch("%s.os.environ" % MODULE_NAME, {}):
      self.assertEqual('basetest/lib', PrepareOptionFiles.getNewLDLibs(None,
                                                                        None,
                                                                        None))

  def test_getnewpath_cornercase(self):
    # TODO: Understand method: Currently this method ignores every path except
    # the last one in the list and just ignores if getSoftwareFolder fails
    reference = os.environ['PATH']
    with patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(side_effect=[S_ERROR(), S_OK('bFolder')])), \
         patch("%s.resolveDeps" % MODULE_NAME, new=Mock(return_value=[self.dep1, self.dep2])), \
         patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[True, False, False, True])), \
         patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=True)):
      self.assertEqual("%s:%s" % ('bFolder/bin', reference),
                         PrepareOptionFiles.getNewPATH(None, None, None))

  def test_getnewpath_nochange(self):
    reference = os.environ['PATH']
    with patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(side_effect=[S_ERROR(), S_OK(''), S_OK('')])), \
         patch("%s.resolveDeps" % MODULE_NAME, new=Mock(return_value=[self.dep1, self.dep2, self.dep3])), \
         patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[False, False])), \
         patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=True)):
      self.assertEqual(reference, PrepareOptionFiles.getNewPATH(None, None, None))

  def test_getnewpath_nokey(self):
    with patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(side_effect=[S_ERROR(), S_OK("testfolder"), S_ERROR()])), \
        patch("%s.resolveDeps" % MODULE_NAME, new=Mock(return_value=[self.dep1, self.dep2, self.dep3])), \
        patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[True, False])), \
        patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=True)), \
        patch("%s.os.environ" % MODULE_NAME, {}):
      self.assertEqual("testfolder/bin",
                         PrepareOptionFiles.getNewPATH(None, None, None))

  def test_prepareWhizFile(self):
    file_contents = [['asdseed123', '314s.sqrtsfe89u', 'n_events143417',
                        'write_events_file', 'processidprocess_id"123',
                        '98u243jrui4fg4289fjh2487rh13urhi'], []]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(), create=True) as file_mocker:
      file_mocker.side_effect = (h for h in handles)
      result = PrepareOptionFiles.prepareWhizardFile("in", "typeA", "1tev",
                                                     "89741", "50", False, "out")
      assertDiracSucceedsWith_equals(result, True, self)
    tuples = [('in', 'r'), ('out', 'w')]
    expected = [[], [' seed = 89741\n', ' sqrts = 1tev\n', ' n_events = 50\n',
                       ' write_events_file = "typeA" \n', 'processidprocess_id"123',
                       '98u243jrui4fg4289fjh2487rh13urhi']]
    FileUtil.checkFileInteractions(self, file_mocker, tuples, expected, handles)

  def test_prepareWhizFile_noprocessid(self):
    file_contents = [['asdseed123', '314s.sqrtsfe89u', 'n_events143417',
                        'write_events_file', 'processidprocess_id1"',
                        '98u243jrui4fg4289fjh2487rh13urhi'], []]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(), create=True) as file_mocker:
      file_mocker.side_effect = (h for h in handles)
      result = PrepareOptionFiles.prepareWhizardFile("in", "typeA", "1tev",
                                                      "89741", "50", False, "out")
      assertDiracSucceedsWith_equals(result, False, self)
    tuples = [('in', 'r'), ('out', 'w')]
    expected = [[], [' seed = 89741\n', ' sqrts = 1tev\n', ' n_events = 50\n',
                       ' write_events_file = "typeA" \n', 'processidprocess_id1"',
                       '98u243jrui4fg4289fjh2487rh13urhi']]
    FileUtil.checkFileInteractions(self, file_mocker, tuples, expected, handles)

  def test_prepareWhizFile_luminosity(self):
    file_contents = ['asdseed123', '314s.sqrtsfe89u', 'n_events143417',
                      'write_events_file', 'processidprocess_id"123',
                      '98u243jrui4fg4289fjh2487rh13urhi', 'luminosity']
    text_file_data = '\n'.join(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(read_data=text_file_data), create=True) as file_mocker:
      result = PrepareOptionFiles.prepareWhizardFile("in", "typeA", "1tev",
                                                      "89741", "50", "684",
                                                      "out")
      assertDiracSucceedsWith_equals(result, True, self)
    file_mocker.assert_any_call('in', 'r')
    file_mocker.assert_any_call('out', 'w')
    mocker_handle = file_mocker()
    expected = [' seed = 89741\n', ' sqrts = 1tev\n', 'n_events143417\n',
                ' write_events_file = "typeA" \n', 'processidprocess_id"123\n',
                '98u243jrui4fg4289fjh2487rh13urhi\n', ' luminosity = 684\n']
    for entry in expected:
      mocker_handle.write.assert_any_call(entry)
    assertEqualsImproved(len(expected),
                          mocker_handle.__enter__.return_value.write.call_count,
                          self)

  def test_prepareWhizFileTemplate(self):
    parameters = create_testdict()
    file_contents = [x + x for x in parameters.keys()]  # Fill file contents with template strings
    # Template strings are the keys of the parameter dictionary concatenated
    # with themselves, e.g. SEEDSEED for the entry 'SEED' : 135431
    parameters['USERSPECTRUM'] = 'mode1234'
    file_contents += ['USERSPECTRUMB1', 'USERSPECTRUMB2']
    file_contents += ['write_events_file', 'processidaisuydhprocess_id"35',
                       'efiuhifuoejf', '198734y37hrunffuydj82']
    text_file_data = '\n'.join(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(read_data=text_file_data), create=True) as file_mocker:
      result = PrepareOptionFiles.prepareWhizardFileTemplate("in", "typeA", parameters, "out")
      assertDiracSucceedsWith_equals(result, True, self)
    file_mocker.assert_any_call('in', 'r')
    file_mocker.assert_any_call('out', 'w')
    mocker_handle = file_mocker()
    expected = [' seed = 135431\n', ' sqrts = 1tev\n', ' beam_recoil = 134\n',
                ' n_events = 23\n', ' luminosity=13\n', ' keep_initials = JE\n',
                " particle_name = 'electron_hans'\n", " particle_name = 'proton_peter'\n",
                ' polarization = plus\n', ' polarization = minus\n', ' USER_spectrum_on = spectrumA\n',
                ' USER_spectrum_on = SpectrumB\n', ' USER_spectrum_mode = mode1234\n',
                ' USER_spectrum_mode = -mode1234\n', ' ISR_on = PSDL\n', ' ISR_on = FVikj\n',
                ' EPA_on = 234\n', ' EPA_on = asf31\n', ' write_events_file = "typeA" \n',
                'processidaisuydhprocess_id"35\n', 'efiuhifuoejf\n', '198734y37hrunffuydj82']
    for entry in expected:
      mocker_handle.write.assert_any_call(entry)
    assertEqualsImproved(len(expected),
                          mocker_handle.__enter__.return_value.write.call_count,
                          self)

  def test_prepareWhizFileTemplate_noprocessid(self):
    parameters = create_testdict()
    file_contents = [x + x for x in parameters.keys()]  # Fill file contents with template strings
    # Template strings are the keys of the parameter dictionary concatenated
    # with themselves, e.g. SEEDSEED for the entry 'SEED' : 135431
    parameters['USERSPECTRUM'] = 'mode1234'
    file_contents += ['USERSPECTRUMB1', 'USERSPECTRUMB2']
    file_contents += ['write_events_file', 'efiuhifuoejf', 'processidprocess_id1""', '198734y37hrunffuydj82']
    text_file_data = '\n'.join(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(read_data=text_file_data), create=True) as file_mocker:
      result = PrepareOptionFiles.prepareWhizardFileTemplate("in", "typeA",
                                                              parameters, "out")
      assertDiracSucceedsWith_equals(result, False, self)
    file_mocker.assert_any_call('in', 'r')
    file_mocker.assert_any_call('out', 'w')
    mocker_handle = file_mocker()
    expected = [' seed = 135431\n', ' sqrts = 1tev\n', ' beam_recoil = 134\n',
                 ' n_events = 23\n', ' luminosity=13\n', ' keep_initials = JE\n',
                 " particle_name = 'electron_hans'\n", " particle_name = 'proton_peter'\n",
                 ' polarization = plus\n', ' polarization = minus\n',
                 ' USER_spectrum_on = spectrumA\n', ' USER_spectrum_on = SpectrumB\n',
                 ' USER_spectrum_mode = mode1234\n', ' USER_spectrum_mode = -mode1234\n',
                 ' ISR_on = PSDL\n', ' ISR_on = FVikj\n', ' EPA_on = 234\n',
                 ' EPA_on = asf31\n', ' write_events_file = "typeA" \n', 'efiuhifuoejf\n',
                 'processidprocess_id1""\n',
                 '198734y37hrunffuydj82']
    for entry in expected:
      mocker_handle.write.assert_any_call(entry)
    assertEqualsImproved(len(expected),
                          mocker_handle.__enter__.return_value.write.call_count,
                          self)

  @patch("%s.open" % MODULE_NAME, mock_open(), create=True)
  def test_prepareXMLFile(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = createXMLTreeForXML().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareXMLFile(
          'finalxml', 'inputxml', 'inputGEAR', ['input slcio file list'], 1,
          'outputfile', 'outputREC', 'outputdst', True)
      assertDiracSucceedsWith_equals(result, True, self)
      mytree = TestPrepareOptionsFile.currenttree
      params = mytree.findall('global/parameter')
      expected_params = ['LCIOInputFiles', 'MaxRecordNumber', 'GearXMLFile',
                          'GearXMLFile', 'Verbosity']
      assertEqualsImproved(len(params), len(expected_params), self)
      for param, expected in zip(params, expected_params):
        assertEqualsImproved(param.attrib['name'], expected, self)
      expected_comments = ['input gear changed', 'input gear changed',
                            'MaxRecordNumber changed', 'input file list changed']
      for child in params:
        if child.attrib['name'] == 'GearXMLFile' and 'value' in child.attrib:
          self.assertEqual(child.attrib['value'], 'inputGEAR')
      glob = mytree.find('global')
      element_counter = 0
      for comment, expected in zip(glob, expected_comments):
        element_counter = element_counter + 1
        assertEqualsImproved(comment.text, expected, self)
      assertEqualsImproved(element_counter, len(expected_comments), self)
      procs = mytree.findall('processor')
      procs = [x for x in procs if x.attrib['name'] == 'MyLCIOOutputProcessor']  # Only get the MyLCIOOutputProcessor
      for proc in procs:
        procparams = proc.findall('parameter')
        self.assertTrue(procparams[0].text == 'outputfile')
        assertEqualsImproved(proc[0].text, 'output file changed', self)

  @patch("%s.open" % MODULE_NAME, mock_open(), create=True)
  @patch("ILCDIRAC.Core.Utilities.MarlinXML.getOverlayFiles", new=Mock(return_value=[]))
  def test_prepareXMLFile_getoverlayEmpty(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = createXMLTreeForXML(1).getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareXMLFile(
          'finalxml', 'inputxml', 'inputGEAR', ['input slcio file list'], 1,
          'outputfile', 'outputREC', 'outputdst', True, overlayParam=[('gghad', 0, None)])
      assertDiracFailsWith(result, 'could not find any overlay files', self)

  @patch("%s.open" % MODULE_NAME, mock_open(), create=True)
  @patch("ILCDIRAC.Core.Utilities.MarlinXML.getOverlayFiles",
         new=Mock(return_value=['overlayfile1', 'verycomplicated_other_file.txt']))
  def test_prepareXMLFile_getoverlayUsed(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = createXMLTreeForXML(2).getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareXMLFile(
          'finalxml', 'inputxml', 'inputGEAR', ['input slcio file list'], 1,
          'outputfile', 'outputREC', 'outputdst', True, overlayParam=[('gghad', 0, None)])
      assertDiracSucceeds(result, self)
      mytree = TestPrepareOptionsFile.currenttree
      procs = mytree.findall('processor')
      procs = [x for x in procs if x.attrib['name'].lower().count(
          'overlaytiming')]  # Only get the one with overlaytiming
      for proc in procs:
        params = proc.findall('parameter')
        self.assertEqual(len(params), 6)
        for i in range(0, 3):
          assertEqualsImproved(params[-1 - i].text,
                                'overlayfile1\nverycomplicated_other_file.txt',
                                self)
          assertEqualsImproved(proc[i].text, 'Overlay files changed', self)

  @patch("%s.ElementTree.parse" % MODULE_NAME, new=Mock(side_effect=IOError))
  def test_prepareXMLFile_parsefails(self):
    result = PrepareOptionFiles.prepareXMLFile(
        'finalxml', 'inputxml', 'inputGEAR', ['input slcio file list'], 1,
        'outputfile', 'outputREC', 'outputdst', True)
    assertDiracFailsWith(result, 'found exception ', self)

  @patch("%s.open" % MODULE_NAME, mock_open(), create=True)
  def test_prepareXMLFile_slciotypeerror(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = createXMLTreeForXML().getroot()
    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareXMLFile(
          'finalxml', 'inputxml', 'inputGEAR', 1, 1, 'outputfile',
          'outputREC', 'outputdst', True)
      assertDiracFailsWith(result, 'inputslcio is neither string nor list!',
                            self)
      assertDiracFailsWith(result, ["actual type is <type 'int'>",  # python2
                                    "actual type is <class 'int'>",  # python3
                                    ],
                           self)

  @patch("%s.open" % MODULE_NAME, mock_open(), create=True)
  def test_prepareXMLFile_nodebug(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = createXMLTreeForXML(-1).getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareXMLFile(
          'finalxml', 'inputxml', 'inputGEAR', 'input slcio file list', 1, '',
          'outputREC', 'outputdst', False)
      assertDiracSucceedsWith_equals(result, True, self)

  def test_prepareSteeringFile_full(self):
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [[], [
        "/Mokka/init/initialMacroFile", "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
        "asdioj/Mokka/init/randomSeedasdki", "13490ielcioFilename12894eu14",
        "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i", "9d0i3198ji31i",
        "nextline", "901-l[doc,193dkdnfba"], []]
    args = ['input.intest', 'output.outtest', "TestdetectormodelClicv302",
             "stdhepfiletest", "", 41, 2, 561351, 8654]
    tuples = [('mokkamac.mac', 'w'), ('input.intest', 'r'),
               ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [['/generator/generator stdhepfiletest\n', '/run/beamOn 41\n'],
                 [], ['9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
                       '#Set detector model to value specified\n',
                       '/Mokka/init/detectorModel TestdetectormodelClicv302\n',
                       '#Set debug level to 1\n', '/Mokka/init/printLevel 1\n',
                       '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
                       '#Set mac file to the one created on the site\n',
                       '/Mokka/init/initialMacroFile mokkamac.mac\n',
                       '#Setting random seed\n', '/Mokka/init/randomSeed 561351\n',
                       "13490ielcioFilename12894eu14", '#Setting run number, same as seed\n',
                       '/Mokka/init/mcRunNumber 8654\n',
                       '#Set event start number to value given as job parameter\n',
                       '/Mokka/init/startEventNumber 2\n']]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples, expected,
                                          exp_retval)

  def test_prepareSteeringFile_nostdhepfile(self):
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [[], ["/Mokka/init/initialMacroFile",
                            "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
                            "asdioj/Mokka/init/randomSeedasdki",
                            "13490ielcioFilename12894eu14",
                            "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i",
                            "9d0i3198ji31i", "nextline",
                            "901-l[doc,193dkdnfba"], []]
    args = ['input.intest', 'output.outtest', "TestdetectormodelClicv302", "",
             "", 41, 2, 561351, 8654, None, True]
    tuples = [('mokkamac.mac', 'w'), ('input.intest', 'r'),
               ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [['/run/beamOn 41\n'], [], [
        '9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
        '#Set detector model to value specified\n',
        '/Mokka/init/detectorModel TestdetectormodelClicv302\n',
        '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
        '#Set mac file to the one created on the site\n',
        '/Mokka/init/initialMacroFile mokkamac.mac\n',
        '#Setting random seed\n', '/Mokka/init/randomSeed 561351\n',
        "13490ielcioFilename12894eu14", '#Setting run number, same as seed\n',
        '/Mokka/init/mcRunNumber 8654\n',
        '#Set event start number to value given as job parameter\n',
        '/Mokka/init/startEventNumber 2\n']]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples,
                                          expected, exp_retval)

  def test_prepareSteeringFile_othercases(self):
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [["/Mokka/init/initialMacroFile",
                        "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
                        "asdioj/Mokka/init/randomSeedasdki", "13490ielcioFilename12894eu14",
                        "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i",
                        "9d0i3198ji31i", "nextline", "901-l[doc,193dkdnfba"], []]
    metadict = {'GenProcessID': 'testgenID', 'CrossSection': '1.3',
                 'Energy': '186', 'PolarizationB1': '', 'PolarizationB2': 'L'}
    args = ['input.intest', 'output.outtest', "TestdetectormodelClicv302",
             "stdhepfiletest", "testmacname", 41, 2, 561351, 8654,
             "testprocessid", False, "testslciooutput.te", metadict]
    tuples = [('input.intest', 'r'), ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [[], ['8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i',
                       '9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
                       '#Set detector model to value specified\n',
                       '/Mokka/init/detectorModel TestdetectormodelClicv302\n',
                       '#Set debug level to 1\n', '/Mokka/init/printLevel 1\n',
                       '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
                       '#Set mac file to the one created on the site\n',
                       '/Mokka/init/initialMacroFile testmacname\n', '#Setting random seed\n',
                       '/Mokka/init/randomSeed 561351\n', '#Setting run number, same as seed\n',
                       '/Mokka/init/mcRunNumber 8654\n',
                       '#Set event start number to value given as job parameter\n',
                       '/Mokka/init/startEventNumber 2\n', '#Set outputfile name to job specified\n',
                       '/Mokka/init/lcioFilename testslciooutput.te\n',
                       "#Set processID as event parameter\n",
                       "/Mokka/init/lcioEventParameter string Process testprocessid\n",
                       "/Mokka/init/lcioEventParameter float CrossSection_fb 1.3\n",
                       "/Mokka/init/lcioEventParameter float Energy 186.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_ep 0.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_em -1.0\n"]]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples,
                                          expected, exp_retval)

  def test_prepareSteeringFile_polarization1(self):
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [["/Mokka/init/initialMacroFile", "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
                        "asdioj/Mokka/init/randomSeedasdki", "13490ielcioFilename12894eu14",
                        "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i",
                        "9d0i3198ji31i", "nextline", "901-l[doc,193dkdnfba"], []]
    metadict = {'GenProcessID': 'testgenID', 'CrossSection': '9.2',
                 'Energy': '-91', 'PolarizationB1': 'R', 'PolarizationB2': ''}
    args = ['input.intest', 'output.outtest', "TestdetectormodelClicv302", "stdhepfiletest",
             "testmacname", 41, 2, 561351, 8654, '', False, "testslciooutput.te", metadict]
    tuples = [('input.intest', 'r'), ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [[], ['8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i',
                       '9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
                       '#Set detector model to value specified\n',
                       '/Mokka/init/detectorModel TestdetectormodelClicv302\n',
                       '#Set debug level to 1\n', '/Mokka/init/printLevel 1\n',
                       '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
                       '#Set mac file to the one created on the site\n',
                       '/Mokka/init/initialMacroFile testmacname\n',
                       '#Setting random seed\n', '/Mokka/init/randomSeed 561351\n',
                       '#Setting run number, same as seed\n',
                       '/Mokka/init/mcRunNumber 8654\n',
                       '#Set event start number to value given as job parameter\n',
                       '/Mokka/init/startEventNumber 2\n',
                       '#Set outputfile name to job specified\n',
                       '/Mokka/init/lcioFilename testslciooutput.te\n',
                       "#Set processID as event parameter\n",
                       "/Mokka/init/lcioEventParameter string Process testgenID\n",
                       "/Mokka/init/lcioEventParameter float CrossSection_fb 9.2\n",
                       "/Mokka/init/lcioEventParameter float Energy -91.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_ep 1.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_em 0.0\n"]]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples, expected,
                                          exp_retval)

  def test_prepareSteeringFile_polarization2(self):
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [["/Mokka/init/initialMacroFile",
                        "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
                        "asdioj/Mokka/init/randomSeedasdki", "13490ielcioFilename12894eu14",
                        "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i",
                        "9d0i3198ji31i", "nextline", "901-l[doc,193dkdnfba"], []]
    metadict = {'GenProcessID': 'testgenID', 'CrossSection': '5.7', 'Energy': '0',
                 'PolarizationB1': 'L', 'PolarizationB2': 'R'}
    args = ['input.intest', 'output.outtest', "TestdetectormodelClicv302",
             "stdhepfiletest", "testmacname", 41, 2, 561351, 8654,
             "testprocessid", False, "testslciooutput.te", metadict]
    tuples = [('input.intest', 'r'), ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [[], ['8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i',
                      '9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
                      '#Set detector model to value specified\n',
                      '/Mokka/init/detectorModel TestdetectormodelClicv302\n',
                      '#Set debug level to 1\n', '/Mokka/init/printLevel 1\n',
                      '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
                      '#Set mac file to the one created on the site\n',
                      '/Mokka/init/initialMacroFile testmacname\n', '#Setting random seed\n',
                      '/Mokka/init/randomSeed 561351\n', '#Setting run number, same as seed\n',
                      '/Mokka/init/mcRunNumber 8654\n',
                      '#Set event start number to value given as job parameter\n',
                      '/Mokka/init/startEventNumber 2\n',
                      '#Set outputfile name to job specified\n',
                      '/Mokka/init/lcioFilename testslciooutput.te\n',
                      "#Set processID as event parameter\n",
                      "/Mokka/init/lcioEventParameter string Process testprocessid\n",
                      "/Mokka/init/lcioEventParameter float CrossSection_fb 5.7\n",
                      "/Mokka/init/lcioEventParameter float Energy 0.0\n",
                      "/Mokka/init/lcioEventParameter float Pol_ep -1.0\n",
                      "/Mokka/init/lcioEventParameter float Pol_em 1.0\n"]]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples, expected,
                                          exp_retval)

  def test_prepareSteeringFile_polarization3(self):
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [["/Mokka/init/initialMacroFile",
                        "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
                        "asdioj/Mokka/init/randomSeedasdki",
                        "13490ielcioFilename12894eu14",
                        "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i",
                        "9d0i3198ji31i", "nextline", "901-l[doc,193dkdnfba"], []]
    metadict = {'GenProcessID': 'testgenID', 'CrossSection': '0.0',
                 'Energy': '3.7', 'PolarizationB1': 'L13',
                 'PolarizationB2': 'R98'}
    args = ['input.intest', 'output.outtest', "TestdetectormodelClicv302",
             "stdhepfiletest", "testmacname", 41, 2, 561351, 8654,
             "testprocessid", False, "testslciooutput.te", metadict]
    tuples = [('input.intest', 'r'), ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [[], ['8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i',
                       '9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
                       '#Set detector model to value specified\n',
                       '/Mokka/init/detectorModel TestdetectormodelClicv302\n',
                       '#Set debug level to 1\n', '/Mokka/init/printLevel 1\n',
                       '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
                       '#Set mac file to the one created on the site\n',
                       '/Mokka/init/initialMacroFile testmacname\n',
                       '#Setting random seed\n', '/Mokka/init/randomSeed 561351\n',
                       '#Setting run number, same as seed\n',
                       '/Mokka/init/mcRunNumber 8654\n',
                       '#Set event start number to value given as job parameter\n',
                       '/Mokka/init/startEventNumber 2\n',
                       '#Set outputfile name to job specified\n',
                       '/Mokka/init/lcioFilename testslciooutput.te\n',
                       "#Set processID as event parameter\n",
                       "/Mokka/init/lcioEventParameter string Process testprocessid\n",
                       "/Mokka/init/lcioEventParameter float CrossSection_fb 0.0\n",
                       "/Mokka/init/lcioEventParameter float Energy 3.7\n",
                       "/Mokka/init/lcioEventParameter float Pol_ep -0.13\n",
                       "/Mokka/init/lcioEventParameter float Pol_em 0.98\n"]]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples, expected,
                                         exp_retval)

  def test_prepareSteeringFile_outputlcio_det_model1(self):
    # Tests with outputlcio set and detmodel not set
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [["/Mokka/init/initialMacroFile",
                        "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
                        "asdioj/Mokka/init/randomSeedasdki",
                        "13490ielcioFilename12894eu14",
                        "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i",
                        "9d0i3198ji31i", "nextline", "901-l[doc,193dkdnfba"], []]
    metadict = {'GenProcessID': 'testgenID', 'CrossSection': '1.3',
                 'Energy': '186', 'PolarizationB1': '', 'PolarizationB2': 'L'}
    args = ['input.intest', 'output.outtest', '', "stdhepfiletest",
             "testmacname", 41, 2, 561351, 8654, "testprocessid", False,
             "testslciooutput.te", metadict]
    tuples = [('input.intest', 'r'), ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [[], ['8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i',
                       '9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
                       '#Set debug level to 1\n', '/Mokka/init/printLevel 1\n',
                       '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
                       '#Set mac file to the one created on the site\n',
                       '/Mokka/init/initialMacroFile testmacname\n',
                       '#Setting random seed\n', '/Mokka/init/randomSeed 561351\n',
                       '#Setting run number, same as seed\n',
                       '/Mokka/init/mcRunNumber 8654\n',
                       '#Set event start number to value given as job parameter\n',
                       '/Mokka/init/startEventNumber 2\n',
                       '#Set outputfile name to job specified\n',
                       '/Mokka/init/lcioFilename testslciooutput.te\n',
                       "#Set processID as event parameter\n",
                       "/Mokka/init/lcioEventParameter string Process testprocessid\n",
                       "/Mokka/init/lcioEventParameter float CrossSection_fb 1.3\n",
                       "/Mokka/init/lcioEventParameter float Energy 186.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_ep 0.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_em -1.0\n"]]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples,
                                         expected, exp_retval)

  def test_prepareSteeringFile_outputlcio_det_model2(self):
    # Tests with outputlcio set and detmodel not set
    # Any open() call removes the first element of this list and uses it as its content
    file_contents = [["/Mokka/init/initialMacroFile",
                        "ewoqijfoifemf/Mokka/init/BatchModeadsifkojmf",
                        "asdioj/Mokka/init/randomSeedasdki",
                        "13490ielcioFilename12894eu14",
                        "8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i",
                        "9d0i3198ji31i", "nextline", "901-l[doc,193dkdnfba"], []]
    metadict = {'GenProcessID': 'testgenID',
                 'CrossSection': '1.3', 'Energy': '186', 'PolarizationB1': '',
                 'PolarizationB2': 'L'}
    args = ['input.intest', 'output.outtest', '', "stdhepfiletest",
             "testmacname", 41, 2, 561351, 8654, "testprocessid", False, '',
             metadict]
    tuples = [('input.intest', 'r'), ('output.outtest', 'w')]

    # expected[i] is the expected output to file i (files are numbered in the
    # order they are opened in the method that is being tested)
    expected = [[], ['13490ielcioFilename12894eu14',
                       '8r9f2u4jikmelf8/Mokka/init/detectorModelasdiojuaf934i',
                       '9d0i3198ji31i', 'nextline', '901-l[doc,193dkdnfba',
                       '#Set debug level to 1\n', '/Mokka/init/printLevel 1\n',
                       '#Set batch mode to true\n', '/Mokka/init/BatchMode true\n',
                       '#Set mac file to the one created on the site\n',
                       '/Mokka/init/initialMacroFile testmacname\n',
                       '#Setting random seed\n', '/Mokka/init/randomSeed 561351\n',
                       '#Setting run number, same as seed\n',
                       '/Mokka/init/mcRunNumber 8654\n',
                       '#Set event start number to value given as job parameter\n',
                       '/Mokka/init/startEventNumber 2\n',
                       "#Set processID as event parameter\n",
                       "/Mokka/init/lcioEventParameter string Process testprocessid\n",
                       "/Mokka/init/lcioEventParameter float CrossSection_fb 1.3\n",
                       "/Mokka/init/lcioEventParameter float Energy 186.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_ep 0.0\n",
                       "/Mokka/init/lcioEventParameter float Pol_em -1.0\n"]]
    exp_retval = S_OK(True)
    self.helper_test_prepareSteeringFile(file_contents, args, tuples,
                                         expected, exp_retval)

  def helper_test_prepareSteeringFile(self, file_contents, args,
                                       expected_file_tuples, expected,
                                       expected_return_value):
    """Helper function to test prepareSteeringFile.

    :param file_contents: List of lists containing the mocked file contents. i-th element is a list whose j-th element is the j-th line of the file it represents. No \n necessary
    :type file_contents: `python:list`
    :param args: Arguments for the call of prepareSteeringFile
    :type args: `python:list`
    :param expected_file_tuples: List of tuples with the filename and mode of opened files. Has to be in order
    :type expected_file_tuples: `python:list`
    :param expected: The expected output of the file operations. List of lists, the i-th element represents the output to the i-th file. Lines have to end with \n
    :type expected: `python:list`
    :param object expected_return_value: The value the call should return
    """
    assertEqualsImproved(len(file_contents), len(expected), self)

    mymock = Mock()
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(mymock), create=True) as file_mocker:
      file_mocker.side_effect = (h for h in handles)
      args.extend([None] * (13 - len(args)))
      result = PrepareOptionFiles.prepareSteeringFile(*args)
      FileUtil.checkFileInteractions(self, file_mocker, expected_file_tuples,
                                      expected, handles)
    assertEqualsImproved(expected_return_value, result, self)

  current_tree = None

  # @patch("ILCDIRAC.Core.Utilities.PrepareOptionFiles.open", mock_open(), create=True)
  # For ease of testing, assert library method write() is correct and
  # instead traverse the xml tree to check for correctness
  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_1().getroot()

    trackstrat = 'trackstrategy'

    def checkstrat(self, driver):
      elem_to_check = driver.find('strategyFile')
      expected_element = ET.Element('strategyFile')
      expected_element.text = trackstrat
      assertEqualsXml(elem_to_check, expected_element, self)
      return True

    slcio_list = ['list of slcio files', 'anotherEntry.txt']
    jar_list = ['list of', 'jar files']
    amEvents = 1
    cachedir = 'cachedir'
    outputfile = 'outputfile'

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', amEvents, trackstrat, slcio_list, jar_list,
          cachedir, outputfile, 'outputrec', 'outputdst', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      # clear is called
      assertEqualsXml(current_tree.find('inputFiles'), ET.Element('inputFiles'), self)
      # element is created for each entry in slcio list
      xml_file_list = current_tree.findall('inputFiles/file')
      assertEqualsImproved(len(slcio_list), len(xml_file_list), self)
      for (slcio_string, treeelem) in zip(slcio_list, xml_file_list):
        expected_element = ET.Element('file')
        expected_element.text = slcio_string
        assertEqualsXml(treeelem, expected_element, self)
      assertEqualsXml(current_tree.find('classpath'), ET.Element('classpath'), self)
      xml_jar_list = current_tree.findall('classpath/jar')
      assertEqualsImproved(len(jar_list), len(xml_jar_list), self)
      for (jar_string, treeelem) in zip(jar_list, xml_jar_list):
        expected_element = ET.Element('jar')
        expected_element.text = jar_string
        assertEqualsXml(treeelem, expected_element, self)
      nbEvents = current_tree.find('control/numberOfEvents')
      assertEqualsImproved(str(amEvents), nbEvents.text, self)
      cachdir = current_tree.find('control/cacheDirectory')
      assertEqualsImproved(cachdir.text, cachedir, self)
      evInterv = current_tree.findall('drivers/driver/eventInterval')
      for ev_int in evInterv:
        assertEqualsImproved(ev_int.text, '1', self)
      drivers = current_tree.findall('drivers/driver')
      ovNameFound = False
      ofpNameFound = False
      for d in drivers:
        if 'type' in d.attrib:
          self.assertTrue(
              not d.attrib['type'] == 'org.lcsim.recon.tracking.seedtracker.steeringwrappers.SeedTrackerWrapper' or checkstrat(
                  self, d))  # note: short circuit operator
          if d.attrib['type'] == 'org.lcsim.util.OverlayDriver':
            ovNameFound = True
            expected_element = ET.Element('overlayFiles')
            expected_element.text = 'overlaytestfile1\ntestfile2.txt'
            assertEqualsXml(d.find('overlayFiles'), expected_element, self)
          if d.attrib['type'] == 'org.lcsim.util.loop.LCIODriver':
            if d.attrib['name'] == 'Writer':
              ofpNameFound = True
              expected_element = ET.Element('outputFilePath')
              expected_element.text = outputfile
              assertEqualsXml(d.find('outputFilePath'), expected_element, self)
      assertEqualsImproved(ovNameFound, ofpNameFound, self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_RECWriter(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_5().getroot()

    trackstrat = 'trackstrategy'
    slcio_list = ['list of slcio files', 'anotherEntry.txt']
    jar_list = ['list of', 'jar files']
    amEvents = 1
    cachedir = 'cachedir'
    outputrec = 'outputrec'

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', amEvents, trackstrat, slcio_list, jar_list,
          cachedir, 'outputfile', outputrec, 'outputdst', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      drivers = current_tree.findall('drivers/driver')
      flag = False
      print('in test')
      print(drivers)
      for d in drivers:
        if 'type' in d.attrib and d.attrib['type'] == 'org.lcsim.util.loop.LCIODriver' and d.attrib['name'] == 'RECWriter':
          expected_element = ET.Element('outputFilePath')
          expected_element.text = outputrec
          assertEqualsXml(d.find('outputFilePath'), expected_element, self)
          flag = True
      self.assertTrue(flag)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_DSTWriter(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_6().getroot()

    trackstrat = 'trackstrategy'
    slcio_list = ['list of slcio files', 'anotherEntry.txt']
    jar_list = ['list of', 'jar files']
    amEvents = 1
    cachedir = 'cachedir'
    outputdst = 'outputdst'
    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', amEvents, trackstrat, slcio_list, jar_list,
          cachedir, 'outputfile', 'outputrec', outputdst, False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      drivers = current_tree.findall('drivers/driver')
      flag = False
      for d in drivers:
        if 'type' in d.attrib and d.attrib['type'] == 'org.lcsim.util.loop.LCIODriver' and d.attrib['name'] == 'DSTWriter':
          expected_element = ET.Element('outputFilePath')
          expected_element.text = outputdst
          assertEqualsXml(d.find('outputFilePath'), expected_element, self)
          flag = True
      self.assertTrue(flag)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_nooutput(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_7().getroot()

    trackstrat = 'trackstrategy'
    slcio_list = ['list of slcio files', 'anotherEntry.txt']
    jar_list = ['list of', 'jar files']
    amEvents = 1
    cachedir = 'cachedir'
    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', amEvents, trackstrat, slcio_list, jar_list,
          cachedir, 'outputfile', '', '', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      ex = current_tree.find('execute/driver')
      assertEqualsXml(ex, ET.Element('driver', name='Writer'), self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_differentoutput(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_8().getroot()

    trackstrat = 'trackstrategy'
    slcio_list = ['list of slcio files', 'anotherEntry.txt']
    jar_list = ['list of', 'jar files']
    amEvents = 1
    cachedir = 'cachedir'
    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', amEvents, trackstrat, slcio_list, jar_list,
          cachedir, 'outputfile', '', '', False)
      assertDiracSucceedsWith_equals(result, 'LCSIM', self)
      current_tree = TestPrepareOptionsFile.current_tree
      drivers = current_tree.findall('drivers/driver')
      flag = False
      for d in drivers:
        if 'type' in d.attrib and d.attrib['type'] == 'org.lcsim.job.EventMarkerDriver':
          expected_element = ET.Element('marker')
          expected_element.text = 'LCSIM'
          assertEqualsXml(d.find('marker'), expected_element, self)
          flag = True
      self.assertTrue(flag)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  def test_prepareLCSIM_lcsimempty(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_2().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 0, 'trackstrategy', ['list of slcio files'],
          [], '', 'outputfile', 'outputrec', 'outputdst', True)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      # New Elem is created
      assertEqualsXml(TestPrepareOptionsFile.current_tree.find('inputFiles'),
                      ET.Element('inputFiles'), self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=[]))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_overlayempty(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_1().getroot()

    trackstrat = 'trackstrategy'
    slcio_list = ['list of slcio files', 'anotherEntry.txt']
    jar_list = ['list of', 'jar files']
    amEvents = 1
    cachedir = 'cachedir'

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', amEvents, trackstrat, slcio_list, jar_list,
          cachedir, 'outputfile', 'outputrec', 'outputdst', False)
      assertDiracFailsWith(result, 'could not find any overlay files', self)
      # check classpath elem is cleared
      assertEqualsXml(TestPrepareOptionsFile.current_tree.find('classpath'),
                      ET.Element('classpath'), self)

  def test_prepareLCSIM_ioerr(self):
    with patch("%s.ElementTree.parse" % MODULE_NAME, side_effect=IOError('')):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy', ['list of slcio files'],
          ['list of', 'jar files'], 'cachedir', 'outputfile', 'outputrec',
          'outputdst', False)
      assertDiracFailsWith(result, 'found exception', self)

  def test_prepareLCSIM_inputlistempty(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_1().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy', [],
          ['list of', 'jar files'], 'cachedir', 'outputfile', 'outputdst',
          'outputrec', False)
      assertDiracFailsWith(result, 'empty input file list', self)

  def test_prepareLCSIM_emptytree(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_3().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy', ['list of slcio files'],
          ['list of', 'jar files'], 'cachedir', 'outputfile', 'outputrec',
          'outputdst', False)
      assertDiracFailsWith(result, 'invalid lcsim file structure', self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_rarecases(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_9().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy',
          ['list of slcio files', 'anotherEntry.txt'], [], 'cachedir',
          'outputfile', 'outputrec', 'outputdst', True)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      expected_element = ET.Element('numberOfEvents')
      expected_element.text = '1'
      assertEqualsXml(current_tree.find('control/numberOfEvents'), expected_element, self)
      self.assertTrue(current_tree.find('control/verbose').text == 'true')
      self.assertTrue(current_tree.find('control/cacheDirectory').text == 'cachedir')
      drivers = current_tree.findall('drivers/driver')
      flag = False
      for d in drivers:
        if 'type' in d.attrib and d.attrib["type"] == "org.lcsim.job.EventMarkerDriver":
          expected_element = ET.Element('eventInterval')
          expected_element.text = '1'
          assertEqualsXml(d.find('eventInterval'), expected_element, self)
          flag = True
      self.assertTrue(flag)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_evtintervalwrong(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_10().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, '',
          ['list of slcio files', 'anotherEntry.txt'], [], 'cachedir',
          'outputfile', 'outputrec', 'outputdst', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      assertEqualsImproved(current_tree.find("drivers/driver/eventInterval").text,
                           '135851', self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_noexec(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_11().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy',
          ['list of slcio files', 'anotherEntry.txt'], [], 'cachedir',
          'outputfile', 'outputrec', 'outputdst', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      drivers = current_tree.findall('drivers/driver')
      flag = False
      for d in drivers:
        if 'type' in d.attrib and d.attrib['type'] == 'org.lcsim.job.EventMarkerDriver' and d.attrib['name'] == 'evtMarker':
          assertEqualsImproved(d.find('eventInterval').text, '1', self)
          flag = True
      self.assertTrue(flag)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_withexec(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_12().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy',
          ['list of slcio files', 'anotherEntry.txt'], [], 'cachedir',
          'outputfile', 'outputrec', 'outputdst', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      drivers = current_tree.findall('drivers/driver')
      flag = False
      for d in drivers:
        if 'type' in d.attrib and d.attrib['type'] == 'org.lcsim.job.EventMarkerDriver' and d.attrib['name'] == 'evtMarker':
          assertEqualsImproved(d.find('eventInterval').text, '1', self)
          flag = True
      self.assertTrue(flag)
      assertEqualsImproved(current_tree.find('execute/driver').attrib['name'],
                           'evtMarker', self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_prepareLCSIM_noclasspath(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_4().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy',
          ['list of slcio files', 'anotherEntry.txt'], ['jarfile'], 'cachedir',
          'outputfile', 'outputrec', 'outputdst', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)
      current_tree = TestPrepareOptionsFile.current_tree
      assertEqualsXml(current_tree.find('classpath'), ET.Element('classpath'),
                      self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_ERROR('bkgtesterror')))
  def test_prepareLCSIM_bkgfails(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_1().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy',
          ['list of slcio files', 'anotherEntry.txt'], [], 'cachedir',
          'outputfile', 'outputrec', 'outputdst', False)
      self.assertFalse(result['OK'])
      assertEqualsImproved('bkgtesterror', result['Message'], self)

  @patch("%s.getOverlayFiles" % MODULE_NAME, new=Mock(return_value=['overlaytestfile1', 'testfile2.txt']))
  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.allowedBkg" % MODULE_NAME, new=Mock(return_value=S_ERROR('dontthrowme')))
  def test_prepareLCSIM_noovname(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_lcsim_13().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      result = PrepareOptionFiles.prepareLCSIMFile(
          'inputlcsim', 'outputlcsim', 1, 'trackstrategy',
          ['list of slcio files', 'anotherEntry.txt'], [], 'cachedir',
          'outputfile', 'outputrec', 'outputdst', False)
      assertDiracSucceedsWith_equals(result, 'testtext', self)

  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  def test_prepareTomato_stdcase(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_salad_1().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      self.assertTrue(PrepareOptionFiles.prepareTomatoSalad(
          'inputxml', 'outputxml', 'inputslcio', 'ofs', 'coll')['OK'])
      current_tree = TestPrepareOptionsFile.current_tree
      it = current_tree.find('global').iter()
      next(it)
      assertEqualsXml(next(it), ET.Comment("input file list changed"), self)
      assertEqualsXml(next(it), ET.Comment("input file list changed"), self)
      for pa in current_tree.findall('global/parameter'):
        if 'name' in pa.attrib and pa.attrib['name'] == 'LCIOInputFiles':
          assertEqualsImproved(pa.text, 'inputslcio', self)
      params = current_tree.findall('processor/parameter')
      for pa in params:
        if 'name' in pa.attrib:
          if pa.attrib['name'] == 'OutputFile':
            assertEqualsImproved(pa.text, 'ofs', self)
          elif pa.attrib['name'] == 'MCCollectionName':
            assertEqualsImproved(pa.text, 'coll', self)

  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  def test_prepareTomato_othercase(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_salad_2().getroot()

    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified):
      self.assertTrue(PrepareOptionFiles.prepareTomatoSalad(
          'inputxml', 'outputxml', 'inputslcio', 'outputFile', 'collection')['OK'])
      current_tree = TestPrepareOptionsFile.current_tree
      expected_element = ET.Element('parameter', name='LCIOInputFiles')
      expected_element.text = 'inputslcio'
      assertEqualsXml(current_tree.findall('global/parameter')[-1],
                      expected_element, self)

  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  def test_prepareTomato_parsefails(self):
    with patch("%s.ElementTree.parse" % MODULE_NAME, new=Mock(side_effect=IOError(''))):
      result = PrepareOptionFiles.prepareTomatoSalad(
          'inputxml', 'outputxml', 'inputslcio', 'outputFile', 'collection')
      self.assertFalse(result['OK'])
      self.assertIn('found exception', result['Message'].lower())

  @patch("%s.ElementTree.write" % MODULE_NAME, new=Mock(return_value=True))
  def test_prepareTomato_noinputfile(self):
    # pylint: disable=unused-argument
    def parseModified(self, source, parser=None):
      """Exchanges the current xmltree object with the one generated by the method."""
      self._root = xml_salad_2().getroot()

    file_contents = [[]]
    # Means 2 files will be opened, nothing is written to first file, and
    # 'firstlineentry' and 'line100' are written (in different calls and
    # exactly these strings) to the second file. If more/less is written this
    # fails!
    expected = [[TestPrepareOptionsFilePatch.teststr]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch("%s.ElementTree.parse" % MODULE_NAME, new=parseModified), \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo:
      mo.side_effect = (h for h in handles)
      self.assertTrue(PrepareOptionFiles.prepareTomatoSalad(
          None, 'outputxml', 'inputslcio', 'outputFile', 'collection')['OK'])
      current_tree = TestPrepareOptionsFile.current_tree
      expected_element = ET.Element('parameter', name='LCIOInputFiles')
      expected_element.text = 'inputslcio'
      assertEqualsXml(current_tree.findall('global/parameter')[-1],
                      expected_element, self)
      expected_tuples = [('default.xml', 'w')]
      FileUtil.checkFileInteractions(self, mo, expected_tuples, expected,
                                      handles)


def createXMLTreeForXML(flag=0):
  """Creates a XML Tree to test prepareXMLFile()"""
  root = ET.Element("root")
  ex = ET.SubElement(root, 'execute')
  ET.SubElement(ex, 'processor', name='MYOVERLAYTIMING', type='OverlayTiming')
  ET.SubElement(ex, 'processor', name='BGoverlAY', type='Overlay')
  glob = ET.SubElement(root, 'global')
  if flag >= 0:
    ET.SubElement(glob, 'parameter', name='LCIOInputFiles')
  ET.SubElement(glob, 'parameter', name='MaxRecordNumber', value='2')
  ET.SubElement(glob, 'parameter', name='GearXMLFile', value='a')
  ET.SubElement(glob, 'parameter', name='GearXMLFile')
  ET.SubElement(glob, 'parameter', name='Verbosity')
  mlop = ET.SubElement(root, 'processor', name='MyLCIOOutputProcessor')
  ET.SubElement(mlop, 'parameter', name='LCIOOutputFile')
  dst = ET.SubElement(root, 'processor', name='DSTOutput')
  ET.SubElement(dst, 'parameter', name='LCIOOutputFile')
  olt = ET.SubElement(root, 'processor', name='MYOVERLAYTIMING', type='OVERLAYTIMING')
  bgo = ET.SubElement(root, 'processor', name='BGoverlAY', type='OVERLAY')
  nbbgVal = '0.0'
  nbbtVal = '0'
  expbgVal = '0.0'
  if flag >= 1:
    nbbgVal = '2.0'
    nbbtVal = '1.3'
    expbgVal = '4.6'
  ET.SubElement(olt, 'parameter', name='NumberBackground', value=nbbgVal)
  ET.SubElement(olt, 'parameter', name='NBunchtrain', value=nbbtVal)
  if flag >= 2:
    ET.SubElement(olt, 'parameter')
    ET.SubElement(olt, 'parameter', name='BackgroundFileNames')
    tmp = ET.SubElement(olt, 'parameter', name='BackgroundFileNames')
    tmp.text = 'something'
  ET.SubElement(bgo, 'parameter', name='expBG', value=expbgVal)
  ET.SubElement(bgo, 'parameter', name='NBunchtrain', value=nbbtVal)
  ET.SubElement(olt, 'parameter', name='BackgroundFileNames')
  ET.SubElement(bgo, 'parameter', name='InputFileNames')

  result = ET.ElementTree(root)
  TestPrepareOptionsFile.currenttree = result
  return result


stdargs = collections.defaultdict(bool)  # returns false by default


def xml_lcsim_1():
  """Creates standard xml tree."""
  return createXMLTreeForLCSIM(stdargs)


def xml_lcsim_2():
  """Creates xml tree without inputFiles element."""
  customargs = copy.deepcopy(stdargs)
  customargs['noInputFiles'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_3():
  """Creates empty xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['returnEmptyTree'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_4():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['with_classpath'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_5():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['use_rec_writer'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_6():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['use_dst_writer'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_7():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['no_writer'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_8():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['different_marker'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_9():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['with_control_subelems'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_10():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['invalid_nbEvts'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_11():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['with_root_execute'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_12():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['with_evt_interval'] = True
  return createXMLTreeForLCSIM(customargs)


def xml_lcsim_13():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs['with_overlaydriver_overlayname_elem'] = True
  return createXMLTreeForLCSIM(customargs)


def createXMLTreeForLCSIM(flags):
  """Creates a XML Tree to test prepareXMLFile()"""
  if flags['returnEmptyTree']:
    return ET.ElementTree()
  root = ET.Element("root")
  if flags['noInputFiles']:
    ET.SubElement(root, 'inputFiles', testarg1='2', testarg2='deleteMe')
  if not flags['with_classpath']:
    ET.SubElement(root, 'classpath', testarg1='2', testarg2='deleteMe')

  co = ET.SubElement(root, 'control')
  if not flags['with_control_subelems']:
    ve = ET.SubElement(co, 'verbose')
    ve.text = 't'
    cd = ET.SubElement(co, 'cacheDirectory')
    cd.text = 't'

  dr = ET.SubElement(root, 'drivers')
  dri = ET.SubElement(dr, 'driver',
                      type='org.lcsim.recon.tracking.seedtracker.steeringwrappers.SeedTrackerWrapper')
  ET.SubElement(dri, 'strategyFile', testarg1='deletemepls', testparam='143')
  if not flags['with_control_subelems'] and not flags['with_root_execute'] and not flags['with_evt_interval']:
    ev = ET.SubElement(dri, 'eventInterval')
    ev.text = '7'
  if flags['invalid_nbEvts']:
    ev.text = '135851'
  if not flags['with_root_execute']:
    ET.SubElement(root, 'execute')

  if flags['with_control_subelems']:
    ET.SubElement(co, 'numberOfEvents')
    ET.SubElement(dr, 'driver', type='org.lcsim.job.EventMarkerDriver')

  dri1 = ET.SubElement(dr, 'driver',
                       type='org.lcsim.recon.tracking.seedtracker.steeringwrappers.SeedTrackerWrapper')
  ET.SubElement(dri1, 'strategyFile')
  ma = ET.SubElement(dri1, 'marker')
  ma.text = 'testtext'
  dri2 = ET.SubElement(dr, 'driver', type='org.lcsim.util.OverlayDriver')
  if not flags['with_overlaydriver_overlayname_elem']:
    ol = ET.SubElement(dri2, 'overlayName')
    ol.text = 'bkgtestname'
  ET.SubElement(dri2, 'overlayFiles')

  ET.SubElement(dr, 'driver')
  dri4 = ET.SubElement(dr, 'driver', type='org.lcsim.util.loop.LCIODriver',
                       name='Writer')
  ET.SubElement(dri4, 'outputFilePath', testparam1='1', asd='123')

  if flags['use_rec_writer']:
    dri5 = ET.SubElement(dr, 'driver', type='org.lcsim.util.loop.LCIODriver',
                         name='RECWriter')
    ET.SubElement(dri5, 'outputFilePath', testparam1='1', asd='123')

  if flags['use_dst_writer']:
    dri6 = ET.SubElement(dr, 'driver', type='org.lcsim.util.loop.LCIODriver',
                         name='DSTWriter')
    ET.SubElement(dri6, 'outputFilePath', testparam1='1', asd='123')

  if flags['no_writer']:
    dr.remove(dri4)

  if flags['different_marker']:
    dri1.remove(ma)
    ET.SubElement(dr, 'driver', type='org.lcsim.job.EventMarkerDriver')

  result = ET.ElementTree(root)
  TestPrepareOptionsFile.current_tree = result
  return result


def createXMLTreeForSalad(flags):
  """Creates a XML Tree to test prepareTomatoSalad()"""
  root = ET.Element("root")
  glob = ET.SubElement(root, 'global')
  ET.SubElement(glob, 'parameter')
  if not flags[0]:
    ET.SubElement(glob, 'parameter', name='LCIOInputFiles')
    ET.SubElement(glob, 'parameter', name='LCIOInputFiles')
  ET.SubElement(glob, 'parameter', name='testme123')
  ET.SubElement(glob, 'parameter')
  ET.SubElement(glob, 'parameter')
  ET.SubElement(glob, 'parameter')

  proc = ET.SubElement(root, 'processor', type='TomatoProcessor')
  ET.SubElement(root, 'processor')

  ET.SubElement(proc, 'parameter')
  ET.SubElement(proc, 'parameter', name='OutputFile')
  ET.SubElement(proc, 'parameter', name='MCCollectionName')

  result = ET.ElementTree(root)
  TestPrepareOptionsFile.current_tree = result
  return result


def xml_salad_1():
  """Creates standard xml tree."""
  return createXMLTreeForSalad(stdargs)


def xml_salad_2():
  """Creates xml tree."""
  customargs = copy.deepcopy(stdargs)
  customargs[0] = True
  return createXMLTreeForSalad(customargs)


def create_testdict():
  """Returns a dictionary with the appropriate parameters for the prepareWhizFileTemplate test."""
  parameters = {}
  parameters['SEED'] = '135431'
  parameters['ENERGY'] = '1tev'
  parameters['RECOIL'] = '134'
  parameters['NBEVTS'] = '23'
  parameters['LUMI'] = '13'
  parameters['INITIALS'] = 'JE'
  parameters['PNAME1'] = 'electron_hans'
  parameters['PNAME2'] = 'proton_peter'
  parameters['POLAB1'] = 'plus'
  parameters['POLAB2'] = 'minus'
  parameters['USERB1'] = 'spectrumA'
  parameters['USERB2'] = 'SpectrumB'
  parameters['ISRB1'] = 'PSDL'
  parameters['ISRB2'] = 'FVikj'
  parameters['EPAB1'] = '234'
  parameters['EPAB2'] = 'asf31'
  return parameters


if __name__ == "__main__":
  SUITE = unittest.defaultTestLoader.loadTestsFromTestCase(TestPrepareOptionsFile)
  TESTRESULT = unittest.TextTestRunner(verbosity=2).run(SUITE)
  print(TESTRESULT)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Utility functions for DD4hep geometry files etc."""

from __future__ import absolute_import
import os
import tarfile

from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getSoftwareFolder, unzip_file_into_dir

LOG = gLogger.getSubLogger(__name__)


class DD4hepMixin(object):
  """mixin class for DD4hep functionality."""

  def _getDetectorXML(self):
    """returns the path to the detector XML file.

    Checks the Configuration System for the Path to DetectorModels or extracts the input sandbox detector xml files

    :returns: S_OK(PathToXMLFile), S_ERROR
    """
    if os.path.exists(os.path.join(self.detectorModel, self.detectorModel + ".xml")):
      LOG.notice("Found detector model: %s" % os.path.join(self.detectorModel, self.detectorModel + ".xml"))
      return S_OK(os.path.join(self.detectorModel, self.detectorModel + ".xml"))
    elif os.path.exists(self.detectorModel + ".zip"):
      LOG.notice("Found detector model zipFile: %s" % self.detectorModel + ".zip")
      return self._extractZip()
    elif os.path.exists(self.detectorModel + ".tar.gz"):
      LOG.notice("Found detector model tarball: %s" % self.detectorModel + ".tar.gz")
      return self._extractTar()
    elif os.path.exists(self.detectorModel + ".tgz"):
      LOG.notice("Found detector model tarball: %s" % self.detectorModel + ".tgz")
      return self._extractTar(extension=".tgz")

    detectorModels = self.ops.getOptionsDict("/DDSimDetectorModels/%s" % (self.applicationVersion))
    if not detectorModels['OK']:
      LOG.error("Failed to get list of DetectorModels from the ConfigSystem", detectorModels['Message'])
      return S_ERROR("Failed to get list of DetectorModels from the ConfigSystem")

    softwareFolder = getSoftwareFolder(self.platform, self.applicationName, self.applicationVersion)
    if not softwareFolder['OK']:
      return softwareFolder
    softwareRoot = softwareFolder['Value']

    if self.detectorModel in detectorModels['Value']:
      detModelPath = detectorModels['Value'][self.detectorModel]
      if not detModelPath.startswith("/"):
        detModelPath = os.path.join(softwareRoot, detModelPath)
      LOG.info("Found path for DetectorModel %s in CS: %s " % (self.detectorModel, detModelPath))
      return S_OK(detModelPath)

    LOG.error('Detector model %s was not found neither locally nor on the web, exiting' % self.detectorModel)
    return S_ERROR('Detector model was not found')

  def _extractTar(self, extension=".tar.gz"):
    """extract the detector tarball for the detectorModel."""
    try:
      detTar = tarfile.open(self.detectorModel + extension, "r:gz")
      detTar.extractall()
      xmlPath = os.path.abspath(os.path.join(self.detectorModel, self.detectorModel + ".xml"))
      return S_OK(xmlPath)
    except (RuntimeError, OSError, IOError) as e:
      LOG.error("Failed to untar detector model", str(e))
      return S_ERROR("Failed to untar detector model")

  def _extractZip(self):
    """extract the detector zip file for the detectorModel."""
    try:
      LOG.notice("Exracting zip file")
      unzip_file_into_dir(open(self.detectorModel + ".zip", 'rb'), os.getcwd())
      xmlPath = os.path.join(os.getcwd(), self.detectorModel, self.detectorModel + ".xml")
      return S_OK(xmlPath)
    except (RuntimeError, OSError, IOError) as err:  # RuntimeError is for zipfile
      LOG.error('Failed to unzip detector model: ', str(err))
      return S_ERROR('Failed to unzip detector model')



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the ILCDIRAC job module."""

from __future__ import print_function
from __future__ import absolute_import
import types
import unittest

from mock import patch, MagicMock as Mock
from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Job import Job
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertInImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith_equals

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Job'

# pylint: disable=protected-access,too-many-public-methods


class JobTestCase(unittest.TestCase):
  """Test the ILCDIRAC Job class."""

  def setUp(self):
    """set up the objects."""
    import DIRAC
    objectLoaderInst = Mock(name="ObjectLoader")
    objectLoaderInst.loadObject.return_value = S_OK(lambda: S_OK(['x86_64-slc5-gcc43-opt']))
    olMock = Mock(name="ObjectLoaderModule", return_value=objectLoaderInst)
    with patch.object(DIRAC.Interfaces.API.Job, 'ObjectLoader', new=olMock):
      self.job = Job('')
    self.job.check = True

  def test_setsysconf(self):
    self.job.getDIRACPlatforms = Mock(return_value=S_OK(['myTestPlatform']))
    self.job.setPlatform('myTestPlatform')
    self.assertFalse(self.job.errorDict)

  def test_unimplemented_methods(self):
    self.job.setInputData('')
    self.job.setInputSandbox('')
    self.job.setOutputData('')
    self.job.setOutputSandbox('')
    self.job.submit()
    assertEqualsImproved(len(self.job.errorDict), 5, self)
    expected_failures = ['setInputData', 'setInputSandbox', 'setOutputData', 'setOutputSandbox', 'submit']
    for method in expected_failures:
      assertInImproved(method, list(self.job.errorDict.keys()), self)

  def test_ignoreapperrors(self):
    assertDiracSucceeds(self.job.setIgnoreApplicationErrors(), self)
    assertEqualsImproved(self.job.workflow.parameters[-1].name, 'IgnoreAppError', self)

  def test_checkparams(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolTestApp'
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkWorkflowConsistency.return_value = S_OK('')
    app_mock._addParametersToStep.return_value = S_OK('')
    app_mock._setStepParametersValues.return_value = S_OK('')
    app_mock._resolveLinkedStepParameters.return_value = S_OK('')
    dirac_mock = Mock()
    dirac_mock.checkparams.return_value = S_OK('dirac_test_retval')
    self.job.applicationlist = [app_mock]
    assertDiracSucceedsWith_equals(self.job.checkparams(dirac_mock), 'dirac_test_retval', self)
    assertEqualsImproved(self.job.workflow.parameters[-1].name, 'TotalSteps', self)

  def test_checkparams_nodirac(self):
    assertDiracFailsWith(self.job.checkparams(), 'missing dirac', self)

  def test_checkparams_analyse_fails(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolTestApp'
    app_mock._analyseJob.return_value = S_ERROR('analyse_test_Err')
    dirac_mock = Mock()
    dirac_mock.checkparams.return_value = S_OK('dirac_test_retval')
    self.job.applicationlist = [app_mock]
    assertDiracFailsWith(self.job.checkparams(dirac_mock), 'analyse_test_err', self)

  def test_checkparams_consistencycheck_fails(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolTestApp'
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkWorkflowConsistency.return_value = S_ERROR('consistency_check_fails')
    dirac_mock = Mock()
    dirac_mock.checkparams.return_value = S_OK('dirac_test_retval')
    self.job.applicationlist = [app_mock]
    assertDiracFailsWith(self.job.checkparams(dirac_mock), 'failed to check its consistency', self)

  def test_checkparams_jobspecificmodules_fails(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolTestApp'
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkWorkflowConsistency.return_value = S_OK('')
    app_mock._addParametersToStep.side_effect = IOError('dont call me')
    app_mock._userjobmodules.return_value = S_ERROR('test_err')
    dirac_mock = Mock()
    dirac_mock.checkparams.return_value = S_OK('dirac_test_retval')
    self.job.applicationlist = [app_mock]
    assertDiracFailsWith(self.job.checkparams(dirac_mock), 'failed to add module', self)

  def test_checkparams_addparam_fails(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolTestApp'
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkWorkflowConsistency.return_value = S_OK('')
    app_mock._addParametersToStep.return_value = S_ERROR('adding of parameters failed')
    dirac_mock = Mock()
    dirac_mock.checkparams.return_value = S_OK('dirac_test_retval')
    self.job.applicationlist = [app_mock]
    assertDiracFailsWith(self.job.checkparams(dirac_mock), 'failed to add parameters', self)

  def test_checkparams_resolveparams_fails(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolTestApp'
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkWorkflowConsistency.return_value = S_OK('')
    app_mock._addParametersToStep.return_value = S_OK('')
    app_mock._setStepParametersValues.return_value = S_ERROR('resolving_failed_testerr')
    dirac_mock = Mock()
    dirac_mock.checkparams.return_value = S_OK('dirac_test_retval')
    self.job.applicationlist = [app_mock]
    assertDiracFailsWith(self.job.checkparams(dirac_mock), 'failed to resolve parameters value', self)

  def test_checkparams_resolvelinks_fails(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolTestApp'
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkWorkflowConsistency.return_value = S_OK('')
    app_mock._addParametersToStep.return_value = S_OK('')
    app_mock._setStepParametersValues.return_value = S_OK('')
    app_mock._resolveLinkedStepParameters.return_value = S_ERROR('resolve_links_failed')
    dirac_mock = Mock()
    dirac_mock.checkparams.return_value = S_OK('dirac_test_retval')
    self.job.applicationlist = [app_mock]
    assertDiracFailsWith(self.job.checkparams(dirac_mock), 'failed to resolve linked parameters', self)

  def test_askuser(self):
    with patch('%s.promptUser' % MODULE_NAME, new=Mock(return_value=S_OK(''))):
      self.job.applicationlist = [Mock()]
      assertDiracSucceeds(self.job._askUser(), self)

  def test_askuser_nocheck(self):
    with patch('%s.promptUser' % MODULE_NAME, new=Mock(return_value=S_ERROR(''))):
      self.job.check = False
      assertDiracSucceeds(self.job._askUser(), self)

  def test_askuser_novalidation(self):
    self.job.applicationlist = []
    with patch('%s.promptUser' % MODULE_NAME, new=Mock(return_value=S_ERROR(''))):
      assertDiracFailsWith(self.job._askUser(), 'user did not validate', self)

  def test_askuser_validation_denied(self):
    self.job.applicationlist = [Mock(), Mock()]
    with patch('%s.promptUser' % MODULE_NAME, new=Mock(return_value=S_OK('n'))):
      assertDiracFailsWith(self.job._askUser(), 'user did not validate', self)

  def test_append(self):
    app_mock = Mock()
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkConsistency.return_value = S_OK('')
    app_mock._checkFinalConsistency.return_value = S_OK('')
    assertDiracSucceeds(self.job.append(app_mock), self)

  def test_append_analysis_fails(self):
    app_mock = Mock()
    app_mock._analyseJob.return_value = S_ERROR('analysis failed, sorry. this is a test')
    assertDiracFailsWith(self.job.append(app_mock), 'analysis failed, sorry', self)

  def test_append_checkconsistency_fails(self):
    app_mock = Mock()
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkConsistency.return_value = S_ERROR('consistency check test failed')
    assertDiracFailsWith(self.job.append(app_mock), 'failed to check its consistency', self)

  def test_append_finalconsistency_fails(self):
    app_mock = Mock()
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkConsistency.return_value = S_OK('')
    app_mock._checkFinalConsistency.return_value = S_ERROR('final consistency invalid')
    assertDiracFailsWith(self.job.append(app_mock), 'failed to check its consistency', self)

  def test_append_jobspecificenergy_wrong(self):
    self.job.energy = 2451
    app_mock = Mock()
    app_mock.energy = 214
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkConsistency.return_value = S_OK('')
    app_mock._checkFinalConsistency.side_effect = IOError('dont call me')
    assertDiracFailsWith(self.job.append(app_mock), 'failed job specific checks', self)

  def test_append_other_case(self):
    app_mock = Mock()
    app_mock.numberOfEvents = 0
    app_mock.appname = ''
    app_mock.inputSB = ['inputsandbox1TestMe', 'other_sandbox', '']
    self.job.inputsandbox = ['other_sandbox']
    app_mock._analyseJob.return_value = S_OK('')
    app_mock._checkConsistency.return_value = S_OK('')
    app_mock._checkFinalConsistency.return_value = S_OK('')
    assertDiracSucceeds(self.job.append(app_mock), self)
    assertEqualsImproved(self.job.inputsandbox, ['other_sandbox', 'inputsandbox1TestMe', ''], self)

  def test_addsoftware(self):
    param_mock = Mock()
    param_mock.getValue.return_value = 'myapp;myappnamtest.testv3/2'
    with patch.object(self.job.workflow, 'findParameter', new=Mock(return_value=param_mock)):
      self.job._addSoftware('myAppNamTest', 'testv3/2')
      assertEqualsImproved(self.job.workflow.parameters[-1].name, 'SoftwarePackages', self)

  def test_addsoftware_addApp(self):
    param_mock = Mock()
    param_mock.getValue.return_value = 'notMyApp'
    with patch.object(self.job.workflow, 'findParameter', new=Mock(return_value=param_mock)):
      self.job._addSoftware('myAppNamTest', 'testv3/2')
      assertEqualsImproved(self.job.workflow.parameters[-1].name, 'SoftwarePackages', self)

# pylint: disable=protected-access


class InternalJobTestCase(unittest.TestCase):
  """Test the methods of the Job class that require a IntrospectJob instance."""

  def setUp(self):
    """set up the objects."""
    import DIRAC
    with patch.object(DIRAC.ConfigurationSystem.Client.Helpers.Resources, 'getDIRACPlatforms', return_value=S_OK(['x86_64-slc5-gcc43-opt'])):
      self.job = IntrospectJob('')
      self.job.check = True

  def test_checkargs_1(self):
    self.job.indirection_for_checkArgs(246, int)
    self.assertFalse(self.job.errorDict)

  def test_checkargs_2(self):
    self.job.indirection_for_checkArgs('bla', int)
    self.assertTrue(self.job.errorDict)

  def test_checkargs_3(self):
    self.job.indirection_for_checkArgs({True: 'blabla'}, dict)
    self.assertFalse(self.job.errorDict)

  def test_checkargs_4(self):
    self.job.indirection_for_checkArgs(False, dict)
    self.assertTrue(self.job.errorDict)

  def test_checkargs_5(self):
    self.job.indirection_for_checkArgs(True, bool)
    self.assertFalse(self.job.errorDict)

  def test_checkargs_6(self):
    self.job.indirection_for_checkArgs({}, bool)
    self.assertTrue(self.job.errorDict)

  def test_checkargs_7(self):
    self.job.indirection_for_checkArgs([True, 129, ''], list)
    self.assertFalse(self.job.errorDict)

  def test_checkargs_8(self):
    self.job.indirection_for_checkArgs(246, list)
    self.assertTrue(self.job.errorDict)

  def test_checkargs_9(self):
    self.job.indirection_2_for_checkArgs(1, int)
    self.assertTrue(self.job.errorDict)

  def test_getargsdict(self):
    my_arg_dict = self.job.indirection_for_getargsdict(arg1=1, arg2=True, arg3='mystring')
    assertEqualsImproved(my_arg_dict, {'arg1': 1, 'arg2': True, 'arg3': 'mystring'}, self)


class IntrospectJob(Job):
  """Used to easily test the introspective methods (e.g. _checkArgs)"""

  def __init__(self, script=None):
    import DIRAC
    objectLoaderInst = Mock(name="ObjectLoader")
    objectLoaderInst.loadObject.return_value = S_OK(lambda: S_OK(['x86_64-slc5-gcc43-opt']))
    olMock = Mock(name="ObjectLoaderModule", return_value=objectLoaderInst)
    with patch.object(DIRAC.Interfaces.API.Job, 'ObjectLoader', new=olMock):
      super(IntrospectJob, self).__init__(script)
    self.getDIRACPlatforms = Mock(return_value=S_OK(['x86_64-slc5-gcc43-opt']))

  def indirection_for_checkArgs(self, arg_to_check, argtype):
    """Method that uses the _checkArgs method so it can be tested."""
    self._checkArgs({'arg_to_check': argtype})
    print(arg_to_check)

  def indirection_2_for_checkArgs(self, arg_to_check, argtype):
    """Method that uses the _checkArgs method so it can be tested."""
    # Intentional 'typo'
    self._checkArgs({'arg_t_check': argtype})
    print(arg_to_check)

  def indirection_for_getargsdict(self, arg1, arg2, arg3):
    """Method that uses the getArgsDict method so it can be tested."""
    print('%s %s %s' % (arg1, arg2, arg3))
    return self._getArgsDict()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Remove any system library provided in the application tar ball.

:author: sposs
:since: Jan 26, 2011
"""

from __future__ import print_function
from __future__ import absolute_import
__RCSID__ = "$Id$"

import os
from DIRAC import gLogger
LOG = gLogger.getSubLogger(__name__)
FILES_TO_REMOVE = ["libc.so", "libc-2.5", "libm.so", "libpthread.so", "libdl.so", "libstdc++.so", "libgcc_s.so.1"]


def removeLibc(libraryPath):
  """Remove libraries that can be problematic, like libc.so.

  :param str libraryPath: libraryPath to look for libraries to remove
  :returns: True on Success, False in case of error
  """

  LOG.debug("RemoveLibC: Trying to remove these libraries:")
  LOG.debug("RemoveLibC - " + "\nRemoveLibC - ".join(FILES_TO_REMOVE))

  curdir = os.getcwd()
  try:
    os.chdir(libraryPath)
  except OSError:
    return True
  listlibs = os.listdir(os.getcwd())
  for lib in listlibs:
    for lib_to_remove in FILES_TO_REMOVE:
      if lib.count(lib_to_remove):
        try:
          libraryPath = os.getcwd() + os.sep + lib
          LOG.info("RemoveLibC: Trying to remove: %s" % libraryPath)
          os.remove(libraryPath)
        except OSError:
          LOG.error("RemoveLibC: Could not remove", lib)
          os.chdir(curdir)
          return False
  os.chdir(curdir)
  return True


def getLibsToIgnore():
  """ :returns: static list of system libraries """
  return FILES_TO_REMOVE


def main():
  """Main method, executed when this file is executed as a python script."""
  import sys
  if not len(sys.argv) > 1:
    LOG.error("You need to pass the path")
    return 1
  PATH = sys.argv[1]
  LOG.notice("Will remove from %s the files that look like %s" % (PATH, getLibsToIgnore()))

  if not removeLibc(PATH):
    LOG.error("Could not clean libs")
    return 1
  return 0


if __name__ == "__main__":
  exit(main())



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Created on Feb 8, 2012.

:author: Stephane Poss
"""
from __future__ import print_function
from __future__ import absolute_import
__RCSID__ = "$Id$"
# pylint: skip-file
# pylint: disable=C0103
from DIRAC.Core.Base import Script
Script.parseCommandLine()

from ILCDIRAC.Interfaces.API.NewInterface.ProductionJob import ProductionJob
from ILCDIRAC.Interfaces.API.NewInterface.Applications import Whizard, Mokka, Marlin, OverlayInput, StdhepCut, StdhepCutJava
from ILCDIRAC.Interfaces.API.NewInterface.Applications import SLIC, LCSIM, SLICPandora, SLCIOSplit, StdHepSplit
from ILCDIRAC.Interfaces.API.DiracILC import DiracILC

dirac = DiracILC()

# As it's a full chain, we start at generation
# so we need to define the process and the energy
# The rest will be set later. We could also set the process
# and the energy directly in the whizard def, but for clarity
# it's better to do it before, that way we know the very
# essential


def getdicts(process):
  """Create the proper structures to build all the prodcutions for the samples with ee_, ea_ aa_."""
  plist = []
  if process.count("ee_"):
    plist.append({'process': process, 'pname1': 'e1', 'pname2': 'E1', "epa_b1": 'F', "epa_b2": 'F'})
  elif process.count("ea_"):
    plist.append({'process': process, 'pname1': 'e1', 'pname2': 'E1', "epa_b1": 'F', "epa_b2": 'T'})
    plist.append({'process': process, 'pname1': 'e1', 'pname2': 'A', "epa_b1": 'F', "epa_b2": 'F'})
    plist.append({'process': process.replace("ea_", "ae_"), 'pname1': 'e1',
                 'pname2': 'E1', "epa_b1": 'T', "epa_b2": 'F'})
    plist.append({'process': process.replace("ea_", "ae_"), 'pname1': 'A',
                 'pname2': 'E1', "epa_b1": 'F', "epa_b2": 'F'})
  elif process.count("aa_"):
    plist.append({'process': process, 'pname1': 'e1', 'pname2': 'E1', "epa_b1": 'T', "epa_b2": 'T'})
    plist.append({'process': process, 'pname1': 'e1', 'pname2': 'A', "epa_b1": 'T', "epa_b2": 'F'})
    plist.append({'process': process, 'pname1': 'A', 'pname2': 'E1', "epa_b1": 'F', "epa_b2": 'T'})
    plist.append({'process': process, 'pname1': 'A', 'pname2': 'A', "epa_b1": 'F', "epa_b2": 'F'})
  else:
    plist.append({'process': process, 'pname1': 'e1', 'pname2': 'E1', "epa_b1": 'F', "epa_b2": 'F'})
  return plist


## tripleH, Hrecoil, stau, gauginos, Hmass, tt, Htautau, Hmumu, Hee, Hbbccmumu, squarks, LCFITraining, Hgammagamma
# HZgamma Hinclusive ZZfusion, Any, ttH, bb_cc_gg
analysis = 'tripleH'
process = 'tt'
#additional_name = '_neu1_356'
globname = ""
additional_name = ''
energy = 3000.
meta_energy = str(int(energy))

# For meta def
meta = {}
meta['ProdID'] = 1
meta['EvtType'] = process
meta['Energy'] = meta_energy


detectormodel = ''  # Can be ILD_00 (and noting else)

# Here get the prod list: initial particles combinasions
prodlist = getdicts(process)

beamrecoil = True

# Do generation
gen = False

# For cuts
cut = False
if cut:
  additional_name += "_cut"

javacut = True
cutfile = 'cuts_qq_nunu_1400.txt'
seleff = 0.52
n_keep = 500

# DoSplit
activesplitstdhep = False
if activesplitstdhep:
  additional_name += "_gensplit"
nbevtsperfilestdhep = 100

# Do Sim
ild_sim = False
sid_sim = True

# DoSplit
activesplit = False
if activesplit:
  additional_name += "_simsplit"

nbevtsperfile = 200

# Do Reco
ild_rec = False
sid_rec = False
# Do Reco with Overlay
ild_rec_ov = False
sid_rec_ov = True

n_events = 100

model = 'sm'
#model = 'susyStagedApproach'
#model = 'susyStagedApproach'+additional_name
#additional_name = '_'
# NO TAUOLA ADD MSTJ(28)=2 in pythia parameters
whcutdict = {}
#whcutdict = {'e2e2_o':["cut M of 3 within 100 150"]}

additionnalreqs = ''
# additionnalreqs = 'MDCY(25,2)=214;MDCY(25,3)=1;' #for decay of H to bb
# additionnalreqs = 'MDCY(25,2)=213;MDCY(25,3)=1;' #for decay of H to cc
# additionnalreqs = 'MDCY(25,2)=219;MDCY(25,3)=1;' #for decay of H to mumu
# additionnalreqs = 'MWID(25)=2;MDCY(25,2)=223;MDCY(25,3)=1;BRAT(223)=1' #for decay of H to gammagamma
# additionnalreqs = 'MWID(25)=2;MDCY(25,2)=224;MDCY(25,3)=1;BRAT(224)=1;MDCY(23,2)=174;MDCY(23,3)=5' #for decay of H to Zgamma, Z to qq
# additionnalreqs = 'MWID(25)=2;MDCY(25,2)=224;MDCY(25,3)=1;BRAT(224)=1;MWID(23)=2;MDCY(23,2)=182;MDCY(23,3)=6; BRAT(182)=0.3333;BRAT(183)=0.;BRAT(184)=0.3333;BRAT(185)=0.;BRAT(186)=0.333$
# additionnalreqs = 'MDCY(25,3)=5;' #for decay of H to quarks
# additionnalreqs = 'MDCY(25,2)=220; MDCY(25,3)=1;'#for decay of H to tau tau
# additionnalreqs = 'MDCY(23,2)=174; MDCY(23,3)=3;'#for decay of Z to bb
mh = 12000.
mb = 2.9
mc = 0
mmu = 0.10566
mtau = 1.77
# Use gridfiles ?
gridfiles = True
# Set exchange lines to 4 (needed for complex final states)
exchange_lines = False


# Whatever is below is not to be touched...
for proddict in prodlist:
  prod_name = additional_name
  process = proddict['process']

  if energy == 1400.:
    spectrum = 19
  elif energy == 3000.:
    spectrum = 11
  elif energy == 500.:
    spectrum = 13
  elif energy == 350.:
    spectrum = 20
  else:
    print("No spectrum defined, cannot proceed")
    exit(1)

  # Start by defining the whizard application

  wh = Whizard(processlist=dirac.getProcessList())
  wh.setModel(model)
  pdict = {}
  pdict['process_input'] = {}
  pdict['process_input']['process_id'] = proddict['process']
  pdict['process_input']['sqrts'] = energy
  if not model == 'sm':
    pdict['process_input']['input_file'] = "LesHouches.msugra_1"
    pdict['process_input']['input_slha_format'] = 'T'
  if beamrecoil and gen:
    prod_name += "_beamrecoil"
    pdict['process_input']['beam_recoil'] = 'T'

  pdict['integration_input'] = {}
  pdict['integration_input']['calls'] = '1  500000 10  500000  1  1500000'
  if gridfiles:
    pdict['integration_input']['read_grids'] = 'T'
  if exchange_lines:
    pdict['integration_input']['exchange_lines'] = 4
  pdict['simulation_input'] = {}
  pdict['simulation_input']['normalize_weight'] = 'F'
  pdict['simulation_input']['n_events'] = n_events
  pdict['simulation_input']['keep_initials'] = 'T'
  pdict['simulation_input']['events_per_file'] = 500000
  if model != 'sm':
    pdict['simulation_input']['pythia_parameters'] = "PMAS(25,1)=%s; PMAS(25,2)=0.3605E-02; MSTU(22)=20 ;PARJ(21)=0.40000;PARJ(41)=0.11000; PARJ(42)=0.52000; PARJ(81)=0.25000; PARJ(82)=1.90000; MSTJ(11)=3; PARJ(54)=-0.03100; PARJ(55)=-0.00200;PARJ(1)=0.08500; PARJ(3)=0.45000; PARJ(4)=0.02500; PARJ(2)=0.31000; PARJ(11)=0.60000; PARJ(12)=0.40000; PARJ(13)=0.72000;PARJ(14)=0.43000; PARJ(15)=0.08000; PARJ(16)=0.08000; PARJ(17)=0.17000; MSTP(3)=1;IMSS(1)=11; IMSS(21)=71; IMSS(22)=71;%s" % (mh, additionnalreqs)
  else:
    pdict['simulation_input']['pythia_parameters'] = "PMAS(25,1)=%s; PMAS(25,2)=0.3605E-02; MSTU(22)=20 ; MSTJ(28)=2 ;PARJ(21)=0.40000;PARJ(41)=0.11000; PARJ(42)=0.52000; PARJ(81)=0.25000; PARJ(82)=1.90000; MSTJ(11)=3; PARJ(54)=-0.03100; PARJ(55)=-0.00200;PARJ(1)=0.08500; PARJ(3)=0.45000; PARJ(4)=0.02500; PARJ(2)=0.31000; PARJ(11)=0.60000; PARJ(12)=0.40000; PARJ(13)=0.72000;PARJ(14)=0.43000; PARJ(15)=0.08000; PARJ(16)=0.08000; PARJ(17)=0.17000; MSTP(3)=1;%s" % (mh, additionnalreqs)
    pdict['parameter_input'] = {}
  #  pdict['parameter_input']['mmu']=mmu
  #  pdict['parameter_input']['mtau']=mtau
  #  pdict['parameter_input']['mb']=mb
  #  pdict['parameter_input']['mc']=mc
    pdict['parameter_input']['mH'] = mh
  pdict['beam_input_1'] = {}
  pdict['beam_input_1']['particle_name'] = proddict['pname1']
  pdict['beam_input_1']['polarization'] = "0.0 0.0"
  pdict['beam_input_1']['USER_spectrum_on'] = 'T'
  pdict['beam_input_1']['USER_spectrum_mode'] = spectrum
  pdict['beam_input_1']['ISR_on'] = 'T'
  pdict['beam_input_1']['EPA_on'] = proddict['epa_b1']

  pdict['beam_input_2'] = {}
  pdict['beam_input_2']['particle_name'] = proddict['pname2']
  pdict['beam_input_2']['polarization'] = "0.0 0.0"
  pdict['beam_input_2']['USER_spectrum_on'] = 'T'
  pdict['beam_input_2']['ISR_on'] = 'T'
  pdict['beam_input_2']['USER_spectrum_mode'] = -spectrum
  pdict['beam_input_2']['EPA_on'] = proddict['epa_b2']

  prod_name += "_" + proddict['pname1'] + proddict['epa_b1'] + "_" + proddict['pname2'] + proddict['epa_b2']

  wh.setFullParameterDict(pdict)
  # wh.setGlobalEvtType("aa_e3e3nn")
  #process = "aa_e3e3nn"
  if globname:
    wh.setGlobalEvtType(globname)
    process = globname

  if whcutdict:
    wh.setGeneratorLevelCuts(whcutdict)

  if cut:
    wh.willCut()

  if gridfiles:
    wh.usingGridFiles()

  if javacut:
    stdhepc = StdhepCutJava()
    stdhepc.setVersion('1.0')
  else:
    stdhepc = StdhepCut()
    stdhepc.setVersion("V7")
  if cut and not cutfile:
    print("No cut file defined, cannot proceed")
    exit(1)
  stdhepc.setSteeringFile(cutfile)
  stdhepc.setMaxNbEvts(n_keep)
  stdhepc.setSelectionEfficiency(seleff)

  # Split
  stdhepsplit = StdHepSplit()
  stdhepsplit.setVersion("V2")
  stdhepsplit.setNumberOfEventsPerFile(nbevtsperfilestdhep)

  # Simulation ILD
  mo = Mokka()
  mo.setVersion('0706P08')
  # mo.setNbEvts(1000)
  if energy in [500., 375., 350.]:
    mo.setSteeringFile("clic_ild_cdr500.steer")
  elif energy in [3000., 1400.]:
    mo.setSteeringFile("clic_ild_cdr.steer")
  else:
    print('Detector Model for Mokka undefined for this energy')
  if detectormodel == 'ild_00':
    mo.setSteeringFile("ild_00.steer")

  # Simulation SID
  slic = SLIC()
  slic.setVersion('v2r9p8')
  slic.setSteeringFile('defaultClicCrossingAngle.mac')
  slic.setDetectorModel('clic_sid_cdr')

  # Split
  split = SLCIOSplit()
  split.setNumberOfEventsPerFile(nbevtsperfile)

  overlay = OverlayInput()
  overlay.setMachine("clic_cdr")
  overlay.setEnergy(energy)
  overlay.setBkgEvtType("gghad")
  if energy == 500.:
    overlay.setBXOverlay(300)
    overlay.setGGToHadInt(0.3)  # When running at 500geV
    overlay.setDetectorModel("CLIC_ILD_CDR500")
  elif energy == 350.:
    overlay.setBXOverlay(300)
    overlay.setGGToHadInt(0.0464)  # When running at 350geV
    overlay.setDetectorModel("CLIC_ILD_CDR500")
  elif energy == 3000.:
    overlay.setBXOverlay(60)
    overlay.setGGToHadInt(3.2)  # When running at 3TeV
    overlay.setDetectorModel("CLIC_ILD_CDR")
  elif energy == 1400.:
    overlay.setBXOverlay(60)
    overlay.setGGToHadInt(1.3)  # When running at 1.4TeV
    overlay.setDetectorModel("CLIC_ILD_CDR")
  else:
    print("Overlay ILD: No overlay parameters defined for this energy")

  # Reconstruction ILD with overlay
  mao = Marlin()
  mao.setDebug()
  mao.setVersion('v0111Prod')
  if ild_rec_ov:
    if energy == 500.:
      mao.setSteeringFile("clic_ild_cdr500_steering_overlay.xml")
      mao.setGearFile('clic_ild_cdr500.gear')
    elif energy == 350.:
      mao.setSteeringFile("clic_ild_cdr500_steering_overlay_350.0.xml")
      mao.setGearFile('clic_ild_cdr500.gear')
    elif energy == 3000.0:
      mao.setSteeringFile("clic_ild_cdr_steering_overlay_3000.0.xml")
      mao.setGearFile('clic_ild_cdr.gear')
    elif energy == 1400.0:
      mao.setSteeringFile("clic_ild_cdr_steering_overlay_1400.0.xml")
      mao.setGearFile('clic_ild_cdr.gear')
    else:
      print("Marlin: No reconstruction suitable for this energy")

  # Reconstruction w/o overlay
  ma = Marlin()
  ma.setDebug()
  ma.setVersion('v0111Prod')
  if ild_rec:
    if energy in [500., 350.]:
      ma.setSteeringFile("clic_ild_cdr500_steering.xml")
      ma.setGearFile('clic_ild_cdr500.gear')
    elif energy in [3000., 1400.]:
      ma.setSteeringFile("clic_ild_cdr_steering.xml")
      ma.setGearFile('clic_ild_cdr.gear')
    else:
      print("Marlin: No reconstruction suitable for this energy")

  # SID Reco w/o overlay
  lcsim_prepandora = LCSIM()
  lcsim_prepandora.setVersion('CLIC_CDR')
  lcsim_prepandora.setSteeringFile("clic_cdr_prePandora.lcsim")
  lcsim_prepandora.setTrackingStrategy("defaultStrategies_clic_sid_cdr.xml")
  # lcsim_prepandora.setDetectorModel('clic_sid_cdr')
  lcsim_prepandora.setOutputFile("prePandora.slcio")
  lcsim_prepandora.willRunSLICPandora()

  slicpandora = SLICPandora()
  slicpandora.setVersion('CLIC_CDR')
  slicpandora.setDetectorModel('clic_sid_cdr')
  slicpandora.setPandoraSettings("PandoraSettingsSlic.xml")
  slicpandora.getInputFromApp(lcsim_prepandora)
  slicpandora.setOutputFile('pandora.slcio')

  lcsim_postpandora = LCSIM()
  lcsim_postpandora.setVersion('CLIC_CDR')
  lcsim_postpandora.getInputFromApp(slicpandora)
  lcsim_postpandora.setSteeringFile("clic_cdr_postPandoraOverlay.lcsim")
  lcsim_postpandora.setTrackingStrategy("defaultStrategies_clic_sid_cdr.xml")
  # lcsim_postpandora.setDetectorModel('clic_sid_cdr')

  # SID Reco w/o overlay
  overlay_sid = OverlayInput()
  overlay_sid.setMachine("clic_cdr")
  overlay_sid.setEnergy(energy)
  overlay_sid.setBkgEvtType("gghad")
  if energy == 3000.:
    overlay_sid.setBXOverlay(60)
    overlay_sid.setGGToHadInt(3.2)  # When running at 3TeV
  elif energy == 350.:
    overlay.setBXOverlay(300)
    overlay.setGGToHadInt(0.0464)  # When running at 350geV
  elif energy == 1400.:
    overlay_sid.setBXOverlay(60)
    overlay_sid.setGGToHadInt(1.3)  # When running at 1.4TeV
  else:
    print("Overlay SID: No overlay parameters defined for this energy")
  overlay_sid.setDetectorModel("CLIC_SID_CDR")

  lcsim_prepandora_ov = LCSIM()
  lcsim_prepandora_ov.setVersion('CLIC_CDR')
  if energy == 3000.0:
    lcsim_prepandora_ov.setSteeringFile("clic_cdr_prePandoraOverlay_3000.0.lcsim")
  elif energy == 1400.0:
    lcsim_prepandora_ov.setSteeringFile("clic_cdr_prePandoraOverlay_1400.0.lcsim")
  else:
    print("LCSIM: No steering files defined for this energy")

  lcsim_prepandora_ov.setTrackingStrategy("defaultStrategies_clic_sid_cdr.xml")
  # lcsim_prepandora_ov.setDetectorModel('clic_sid_cdr')
  lcsim_prepandora_ov.setOutputFile("prePandora.slcio")
  lcsim_prepandora_ov.willRunSLICPandora()

  slicpandora_ov = SLICPandora()
  slicpandora_ov.getInputFromApp(lcsim_prepandora_ov)
  slicpandora_ov.setVersion('CLIC_CDR')
  slicpandora_ov.setDetectorModel('clic_sid_cdr')
  slicpandora_ov.setPandoraSettings("PandoraSettingsSlic.xml")
  slicpandora_ov.setOutputFile('pandora.slcio')

  lcsim_postpandora_ov = LCSIM()
  lcsim_postpandora_ov.getInputFromApp(slicpandora_ov)
  lcsim_postpandora_ov.setVersion('CLIC_CDR')
  lcsim_postpandora_ov.setSteeringFile("clic_cdr_postPandoraOverlay.lcsim")
  lcsim_postpandora_ov.setTrackingStrategy("defaultStrategies_clic_sid_cdr.xml")
  # lcsim_postpandora_ov.setDetectorModel('clic_sid_cdr')

  if gen:
    ##########################################
    # Define the generation production.
    pwh = ProductionJob()
    pwh.setLogLevel("verbose")
    pwh.setOutputSE("CERN-SRM")
    pwh.setProdType("MCGeneration")
    wname = process + "_" + str(energy)
    if additionnalreqs:
      wname += "_forced_decay"
    if cut:
      wname += "_cut"
    wname += prod_name
    pwh.setWorkflowName(wname)
    pwh.setProdGroup(analysis + "_" + str(energy))
    res = pwh.append(wh)
    if not res['OK']:
      print(res['Message'])
      exit(1)

    if cut:
      res = pwh.append(stdhepc)
      if not res['OK']:
        print(res['Message'])
        exit(1)

    pwh.addFinalization(True, True, True, True)
    descrp = "CLIC %s BeamSpread, ISR ON, whizard" % energy
    if additionnalreqs:
      descrp += ", %s" % additionnalreqs
    if cut:
      descrp += ", cut at stdhep level"
    if prod_name:
      descrp += ", %s" % prod_name

    pwh.setDescription(descrp)

    res = pwh.createProduction()
    if not res['OK']:
      print(res['Message'])

    pwh.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                 "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = pwh.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pwh.setNbOfTasks(1)
    # The production is created, one can now take care of the second step:
    # For that we will use the metadata of the previous production as input
    meta = pwh.getMetadata()

  if activesplitstdhep and meta:
    pstdhepsplit = ProductionJob()
    pstdhepsplit.setLogLevel("verbose")
    pstdhepsplit.setProdType('Split')
    res = pstdhepsplit.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pstdhepsplit.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_split"
    wname += prod_name
    pstdhepsplit.setWorkflowName(wname)
    pstdhepsplit.setProdGroup(analysis + "_" + str(energy))

    # Add the application
    res = pstdhepsplit.append(stdhepsplit)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pstdhepsplit.addFinalization(True, True, True, True)
    descrp = "Splitting stdhep files"

    if prod_name:
      descrp += ", %s" % prod_name

    pstdhepsplit.setDescription(descrp)

    res = pstdhepsplit.createProduction()
    if not res['OK']:
      print(res['Message'])

    pstdhepsplit.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                          "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = pstdhepsplit.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)
    # As before: get the metadata for this production to input into the next
    meta = pstdhepsplit.getMetadata()

  if ild_sim and meta:
    ####################
    # Define the second production (simulation). Notice the setInputDataQuery call
    pmo = ProductionJob()
    pmo.setLogLevel("verbose")
    pmo.setProdType('MCSimulation')
    res = pmo.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pmo.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_ild_sim"
    wname += prod_name
    pmo.setWorkflowName(wname)
    pmo.setProdGroup(analysis + "_" + str(energy))
    # Add the application
    res = pmo.append(mo)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pmo.addFinalization(True, True, True, True)
    if energy > 550.:
      descrp = "CLIC_ILD_CDR model"
    else:
      descrp = "CLIC_ILD_CDR_500 model"
    if prod_name:
      descrp += ", %s" % prod_name
    pmo.setDescription(descrp)
    res = pmo.createProduction()
    if not res['OK']:
      print(res['Message'])

    pmo.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                 "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = pmo.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)
    # As before: get the metadata for this production to input into the next
    meta = pmo.getMetadata()

  if sid_sim and meta:
    ####################
    # Define the second production (simulation). Notice the setInputDataQuery call
    psl = ProductionJob()
    psl.setLogLevel("verbose")
    psl.setProdType('MCSimulation')
    res = psl.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psl.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_sid_sim"
    wname += prod_name
    psl.setWorkflowName(wname)
    psl.setProdGroup(analysis + "_" + str(energy))
    # Add the application
    res = psl.append(slic)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psl.addFinalization(True, True, True, True)
    descrp = "CLIC_SID_CDR model"
    if prod_name:
      descrp += ", %s" % prod_name
    psl.setDescription(descrp)

    res = psl.createProduction()
    if not res['OK']:
      print(res['Message'])

    psl.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                 "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = psl.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)
    # As before: get the metadata for this production to input into the next
    meta = psl.getMetadata()

  if activesplit and meta:
    #######################
    # Split the input files.
    psplit = ProductionJob()
    psplit.setCPUTime(30000)
    psplit.setLogLevel("verbose")
    psplit.setProdType('Split')
    psplit.setDestination("LCG.CERN.ch")
    res = psplit.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psplit.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_split"
    wname += prod_name
    psplit.setWorkflowName(wname)
    psplit.setProdGroup(analysis + "_" + str(energy))

    # Add the application
    res = psplit.append(split)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psplit.addFinalization(True, True, True, True)
    descrp = "Splitting slcio files"
    if prod_name:
      descrp += ", %s" % prod_name
    psplit.setDescription(descrp)

    res = psplit.createProduction()
    if not res['OK']:
      print(res['Message'])
    psplit.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                    "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = psplit.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)
    # As before: get the metadata for this production to input into the next
    meta = psplit.getMetadata()

  if ild_rec and meta:
    #######################
    # Define the reconstruction prod
    pma = ProductionJob()
    pma.setLogLevel("verbose")
    pma.setProdType('MCReconstruction')
    res = pma.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pma.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_ild_rec"
    wname += prod_name
    pma.setWorkflowName(wname)
    pma.setProdGroup(analysis + "_" + str(energy))

    # Add the application
    res = pma.append(ma)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pma.addFinalization(True, True, True, True)
    if energy > 550.:
      descrp = "CLIC_ILD_CDR, No overlay"
    else:
      descrp = "CLIC_ILD_CDR 500 gev, No overlay"
    if prod_name:
      descrp += ", %s" % prod_name
    pma.setDescription(descrp)

    res = pma.createProduction()
    if not res['OK']:
      print(res['Message'])
    pma.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                 "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = pma.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)

  if sid_rec and meta:
    #######################
    # Define the reconstruction prod
    psidrec = ProductionJob()
    psidrec.setLogLevel("verbose")
    psidrec.setProdType('MCReconstruction')
    psidrec.setBannedSites(['LCG.Bristol.uk', 'LCG.RAL-LCG2.uk'])
    res = psidrec.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psidrec.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_sid_rec"
    wname += prod_name
    psidrec.setWorkflowName(wname)
    psidrec.setProdGroup(analysis + "_" + str(energy))
    res = psidrec.append(lcsim_prepandora)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    res = psidrec.append(slicpandora)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    res = psidrec.append(lcsim_postpandora)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psidrec.addFinalization(True, True, True, True)
    descrp = "CLIC_SID_CDR, No overlay"
    if prod_name:
      descrp += ", %s" % prod_name
    psidrec.setDescription(descrp)

    res = psidrec.createProduction()
    if not res['OK']:
      print(res['Message'])

    psidrec.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                     "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = psidrec.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)

  if ild_rec_ov and meta:
    #######################
    # Define the reconstruction prod
    pmao = ProductionJob()
    pmao.setLogLevel("verbose")
    pmao.setProdType('MCReconstruction_Overlay')
    res = pmao.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pmao.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_ild_rec_overlay"
    wname += prod_name
    pmao.setWorkflowName(wname)
    pmao.setProdGroup(analysis + "_" + str(energy))

    # Add the application
    res = pmao.append(overlay)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    # Add the application
    res = pmao.append(mao)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    pmao.addFinalization(True, True, True, True)
    if energy > 550.:
      descrp = "CLIC_ILD_CDR, Overlay"
    else:
      descrp = "CLIC_ILD_CDR_500, Overlay"
    if prod_name:
      descrp += ", %s" % prod_name
    pmao.setDescription(descrp)
    res = pmao.createProduction()
    if not res['OK']:
      print(res['Message'])

    pmao.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                  "EPA_B1": proddict['epa_b1'], "EPA_B2": proddict['epa_b2']})

    res = pmao.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)

  if sid_rec_ov and meta:
    #######################
    # Define the reconstruction prod
    psidreco = ProductionJob()
    psidreco.setLogLevel("verbose")
    psidreco.setProdType('MCReconstruction_Overlay')
    psidreco.setBannedSites(['LCG.Bristol.uk', 'LCG.RAL-LCG2.uk'])
    res = psidreco.setInputDataQuery(meta)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psidreco.setOutputSE("CERN-SRM")
    wname = process + "_" + str(energy) + "_sid_rec_overlay"
    wname += prod_name
    psidreco.setWorkflowName(wname)
    psidreco.setProdGroup(analysis + "_" + str(energy))
    res = psidreco.append(overlay_sid)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    res = psidreco.append(lcsim_prepandora_ov)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    res = psidreco.append(slicpandora_ov)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    res = psidreco.append(lcsim_postpandora_ov)
    if not res['OK']:
      print(res['Message'])
      exit(1)
    psidreco.addFinalization(True, True, True, True)
    descrp = "CLIC_SID_CDR, overlay"
    if prod_name:
      descrp += ", %s" % prod_name
    psidreco.setDescription(descrp)

    res = psidreco.createProduction()
    if not res['OK']:
      print(res['Message'])
    psidreco.addMetadataToFinalFiles({"BeamParticle1": proddict['pname1'], "BeamParticle2": proddict['pname2'],
                                      "EPA_Beam1": proddict['epa_b1'], "EPA_Beam2": proddict['epa_b2']})

    res = psidreco.finalizeProd()
    if not res['OK']:
      print(res['Message'])
      exit(1)

  # In principle nothing else is needed.
